@@ -1,6 +1,4 @@
-# -*- coding: utf-8 -*-
-#
-# Copyright 2010-2017 The pygit2 contributors
+# Copyright 2010-2019 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2,
@@ -25,9 +23,6 @@
 # the Free Software Foundation, 51 Franklin Street, Fifth Floor,
 # Boston, MA 02110-1301, USA.
 
-# Import from the future
-from __future__ import absolute_import
-
 # Import from pygit2
 from .errors import check_error
 from .ffi import ffi, C
@@ -43,12 +38,12 @@ class Refspec(object):
     @property
     def src(self):
         """Source or lhs of the refspec"""
-        return ffi.string(C.git_refspec_src(self._refspec)).decode()
+        return ffi.string(C.git_refspec_src(self._refspec)).decode('utf-8')
 
     @property
     def dst(self):
         """Destinaton or rhs of the refspec"""
-        return ffi.string(C.git_refspec_dst(self._refspec)).decode()
+        return ffi.string(C.git_refspec_dst(self._refspec)).decode('utf-8')
 
     @property
     def force(self):
@@ -58,7 +53,7 @@ class Refspec(object):
     @property
     def string(self):
         """String which was used to create this refspec"""
-        return ffi.string(C.git_refspec_string(self._refspec)).decode()
+        return ffi.string(C.git_refspec_string(self._refspec)).decode('utf-8')
 
     @property
     def direction(self):
@@ -82,9 +77,9 @@ class Refspec(object):
         check_error(err)
 
         try:
-            return ffi.string(buf.ptr).decode()
+            return ffi.string(buf.ptr).decode('utf-8')
         finally:
-            C.git_buf_free(buf)
+            C.git_buf_dispose(buf)
 
     def transform(self, ref):
         """Transform a reference name according to this refspec from the lhs to
