@@ -149,6 +149,23 @@ Branch_branch_name__get__(Branch *self)
         return Error_set(err);
 }
 
+PyDoc_STRVAR(Branch_raw_branch_name__doc__,
+  "The name of the local or remote branch (bytes).");
+
+PyObject *
+Branch_raw_branch_name__get__(Branch *self)
+{
+    int err;
+    const char *c_name;
+
+    CHECK_REFERENCE(self);
+
+    err = git_branch_name(&c_name, self->reference);
+    if (err == GIT_OK)
+        return PyBytes_FromString(c_name);
+    else
+        return Error_set(err);
+}
 
 PyDoc_STRVAR(Branch_remote_name__doc__,
   "The name of the remote set to be the upstream of this branch.");
@@ -169,7 +186,7 @@ Branch_remote_name__get__(Branch *self)
         return Error_set(err);
 
     py_name = to_unicode(name.ptr, NULL, NULL);
-    git_buf_free(&name);
+    git_buf_dispose(&name);
 
     return py_name;
 }
@@ -247,7 +264,7 @@ Branch_upstream_name__get__(Branch *self)
         return Error_set(err);
 
     py_name = to_unicode(name.ptr, NULL, NULL);
-    git_buf_free(&name);
+    git_buf_dispose(&name);
 
     return py_name;
 }
@@ -263,6 +280,7 @@ PyMethodDef Branch_methods[] = {
 
 PyGetSetDef Branch_getseters[] = {
     GETTER(Branch, branch_name),
+    GETTER(Branch, raw_branch_name),
     GETTER(Branch, remote_name),
     GETSET(Branch, upstream),
     GETTER(Branch, upstream_name),
