@@ -73,8 +73,13 @@ wrap_diff_file(const git_diff_file *file)
     py_file = PyObject_New(DiffFile, &DiffFileType);
     if (py_file) {
         py_file->id = git_oid_to_python(&file->id);
-        py_file->path = file->path != NULL ? strdup(file->path) : NULL;
-        py_file->raw_path = file->path != NULL ? PyBytes_FromString(file->path) : NULL;
+        if (file->path) {
+            py_file->path = strdup(file->path);
+            py_file->raw_path = PyBytes_FromString(file->path);
+        } else {
+            py_file->path = NULL;
+            py_file->raw_path = NULL;
+        }
         py_file->size = file->size;
         py_file->flags = file->flags;
         py_file->mode = file->mode;
@@ -169,8 +174,8 @@ static void
 DiffFile_dealloc(DiffFile *self)
 {
     Py_CLEAR(self->id);
-    if (self->path)
-        free(self->path);
+    Py_CLEAR(self->raw_path);
+    free(self->path);
     PyObject_Del(self);
 }
 
@@ -239,12 +244,7 @@ PyObject *
 DiffDelta_status_char(DiffDelta *self)
 {
     char status = git_diff_status_char(self->status);
-
-#if PY_MAJOR_VERSION == 2
-    return Py_BuildValue("c", status);
-#else
     return Py_BuildValue("C", status);
-#endif
 }
 
 PyDoc_STRVAR(DiffDelta_is_binary__doc__, "True if binary data, False if not.");
@@ -347,28 +347,28 @@ PyDoc_STRVAR(DiffLine_old_lineno__doc__, "Line number in old file or -1 for adde
 PyObject *
 DiffLine_old_lineno__get__(DiffLine *self)
 {
-    return PyInt_FromLong(self->line->old_lineno);
+    return PyLong_FromLong(self->line->old_lineno);
 }
 
 PyDoc_STRVAR(DiffLine_new_lineno__doc__, "Line number in new file or -1 for deleted line");
 PyObject *
 DiffLine_new_lineno__get__(DiffLine *self)
 {
-    return PyInt_FromLong(self->line->new_lineno);
+    return PyLong_FromLong(self->line->new_lineno);
 }
 
 PyDoc_STRVAR(DiffLine_num_lines__doc__, "Number of newline characters in content");
 PyObject *
 DiffLine_num_lines__get__(DiffLine *self)
 {
-    return PyInt_FromLong(self->line->num_lines);
+    return PyLong_FromLong(self->line->num_lines);
 }
 
 PyDoc_STRVAR(DiffLine_content_offset__doc__, "Offset in the original file to the content");
 PyObject *
 DiffLine_content_offset__get__(DiffLine *self)
 {
-    return PyInt_FromLongLong(self->line->content_offset);
+    return PyLong_FromLongLong(self->line->content_offset);
 }
 
 PyDoc_STRVAR(DiffLine_content__doc__, "Content of the diff line");
@@ -507,7 +507,7 @@ diff_get_delta_byindex(git_diff *diff, size_t idx)
 {
     const git_diff_delta *delta = git_diff_get_delta(diff, idx);
     if (delta == NULL) {
-        PyErr_SetObject(PyExc_IndexError, PyInt_FromSize_t(idx));
+        PyErr_SetObject(PyExc_IndexError, PyLong_FromSize_t(idx));
         return NULL;
     }
 
@@ -618,10 +618,10 @@ Diff_patch__get__(Diff *self)
     }
 
     py_patch = to_unicode(buf.ptr, NULL, NULL);
-    git_buf_free(&buf);
+    git_buf_dispose(&buf);
 
 cleanup:
-    git_buf_free(&buf);
+    git_buf_dispose(&buf);
     return (err < 0) ? Error_set(err) : py_patch;
 }
 
@@ -638,7 +638,7 @@ PyDoc_STRVAR(DiffHunk_old_start__doc__, "Old start.");
 PyObject *
 DiffHunk_old_start__get__(DiffHunk *self)
 {
-  return PyInt_FromLong(self->hunk->old_start);
+  return PyLong_FromLong(self->hunk->old_start);
 }
 
 PyDoc_STRVAR(DiffHunk_old_lines__doc__, "Old lines.");
@@ -646,7 +646,7 @@ PyDoc_STRVAR(DiffHunk_old_lines__doc__, "Old lines.");
 PyObject *
 DiffHunk_old_lines__get__(DiffHunk *self)
 {
-  return PyInt_FromLong(self->hunk->old_lines);
+  return PyLong_FromLong(self->hunk->old_lines);
 }
 
 PyDoc_STRVAR(DiffHunk_new_start__doc__, "New start.");
@@ -654,7 +654,7 @@ PyDoc_STRVAR(DiffHunk_new_start__doc__, "New start.");
 PyObject *
 DiffHunk_new_start__get__(DiffHunk *self)
 {
-  return PyInt_FromLong(self->hunk->new_start);
+  return PyLong_FromLong(self->hunk->new_start);
 }
 
 PyDoc_STRVAR(DiffHunk_new_lines__doc__, "New lines.");
@@ -662,7 +662,7 @@ PyDoc_STRVAR(DiffHunk_new_lines__doc__, "New lines.");
 PyObject *
 DiffHunk_new_lines__get__(DiffHunk *self)
 {
-  return PyInt_FromLong(self->hunk->new_lines);
+  return PyLong_FromLong(self->hunk->new_lines);
 }
 
 PyDoc_STRVAR(DiffHunk_header__doc__, "Header.");
@@ -760,7 +760,7 @@ PyDoc_STRVAR(DiffStats_insertions__doc__, "Total number of insertions");
 PyObject *
 DiffStats_insertions__get__(DiffStats *self)
 {
-    return PyInt_FromSize_t(git_diff_stats_insertions(self->stats));
+    return PyLong_FromSize_t(git_diff_stats_insertions(self->stats));
 }
 
 PyDoc_STRVAR(DiffStats_deletions__doc__, "Total number of deletions");
@@ -768,7 +768,7 @@ PyDoc_STRVAR(DiffStats_deletions__doc__, "Total number of deletions");
 PyObject *
 DiffStats_deletions__get__(DiffStats *self)
 {
-    return PyInt_FromSize_t(git_diff_stats_deletions(self->stats));
+    return PyLong_FromSize_t(git_diff_stats_deletions(self->stats));
 }
 
 PyDoc_STRVAR(DiffStats_files_changed__doc__, "Total number of files changed");
@@ -776,7 +776,7 @@ PyDoc_STRVAR(DiffStats_files_changed__doc__, "Total number of files changed");
 PyObject *
 DiffStats_files_changed__get__(DiffStats *self)
 {
-    return PyInt_FromSize_t(git_diff_stats_files_changed(self->stats));
+    return PyLong_FromSize_t(git_diff_stats_files_changed(self->stats));
 }
 
 PyDoc_STRVAR(DiffStats_format__doc__,
@@ -816,7 +816,7 @@ DiffStats_format(DiffStats *self, PyObject *args, PyObject *kwds)
         return Error_set(err);
 
     str = to_unicode(buf.ptr, NULL, NULL);
-    git_buf_free(&buf);
+    git_buf_dispose(&buf);
 
     return str;
 }
@@ -977,10 +977,10 @@ Diff_getitem(Diff *self, PyObject *value)
 {
     size_t i;
 
-    if (!PyInt_Check(value))
+    if (!PyLong_Check(value))
         return NULL; /* FIXME Raise error */
 
-    i = PyInt_AsSize_t(value);
+    i = PyLong_AsSize_t(value);
     return diff_get_patch_byindex(self->diff, i);
 }
 
@@ -998,24 +998,24 @@ PyDoc_STRVAR(Diff_parse_diff__doc__,
     "Parses a git unified diff into a diff object without a repository");
 
 static PyObject *
-Diff_parse_diff(PyObject *self, PyObject *args)
+Diff_parse_diff(PyObject *self, PyObject *py_str)
 {
-  /* A wrapper around
-   * git_diff_from_buffer
-  */
-  git_diff *diff;
-  const char *content = NULL;
-  Py_ssize_t content_len;
-  int err;
+    /* A wrapper around git_diff_from_buffer */
+    git_diff *diff;
+    const char *content = NULL;
+    int err;
+    PyObject *tvalue;
 
-  if (!PyArg_ParseTuple(args, "s#", &content, &content_len))
-    return NULL;
+    content = py_str_borrow_c_str(&tvalue, py_str, NULL);
+    if (content == NULL)
+        return NULL;
 
-  err = git_diff_from_buffer(&diff, content, content_len);
-  if (err < 0)
-    return Error_set(err);
+    err = git_diff_from_buffer(&diff, content, strlen(content));
+    Py_DECREF(tvalue);
+    if (err < 0)
+        return Error_set(err);
 
-  return wrap_diff(diff, NULL);
+    return wrap_diff(diff, NULL);
 }
 
 static void
@@ -1044,7 +1044,7 @@ static PyMethodDef Diff_methods[] = {
     METHOD(Diff, find_similar, METH_VARARGS | METH_KEYWORDS),
     METHOD(Diff, from_c, METH_STATIC | METH_VARARGS),
     {"parse_diff", (PyCFunction) Diff_parse_diff,
-      METH_VARARGS | METH_STATIC, Diff_parse_diff__doc__},
+      METH_O | METH_STATIC, Diff_parse_diff__doc__},
     {NULL}
 };
 
