@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2017 The pygit2 contributors
+ * Copyright 2010-2019 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -39,7 +39,7 @@ PyObject* Diff_patch(Diff *self);
 PyObject* wrap_diff(git_diff *diff, Repository *repo);
 PyObject* wrap_diff_delta(const git_diff_delta *delta);
 PyObject* wrap_diff_file(const git_diff_file *file);
-PyObject * wrap_diff_hunk(git_patch *patch, size_t idx);
-PyObject* wrap_diff_line(const git_diff_line *line);
+PyObject* wrap_diff_hunk(Patch *patch, size_t idx);
+PyObject* wrap_diff_line(const git_diff_line *line, DiffHunk *hunk);
 
 #endif
