@@ -31,6 +31,7 @@
 #include "error.h"
 #include "types.h"
 #include "utils.h"
+#include "odb.h"
 #include "oid.h"
 #include "repository.h"
 #include "object.h"
@@ -47,22 +48,46 @@ Object_dealloc(Object* self)
 {
     Py_CLEAR(self->repo);
     git_object_free(self->obj);
+    git_tree_entry_free((git_tree_entry*)self->entry);
     Py_TYPE(self)->tp_free(self);
 }
 
 
+git_object*
+Object__load(Object *self)
+{
+    if (self->obj == NULL) {
+        int err = git_tree_entry_to_object(&self->obj, self->repo->repo, self->entry);
+        if (err < 0) {
+            Error_set(err);
+            return NULL;
+        }
+    }
+
+    return self->obj;
+}
+
+const git_oid*
+Object__id(Object *self)
+{
+    return (self->obj) ? git_object_id(self->obj) : git_tree_entry_id(self->entry);
+}
+
+
+git_object_t
+Object__type(Object *self)
+{
+    return (self->obj) ? git_object_type(self->obj) : git_tree_entry_type(self->entry);
+}
+
+
 PyDoc_STRVAR(Object_id__doc__,
     "The object id, an instance of the Oid type.");
 
 PyObject *
 Object_id__get__(Object *self)
 {
-    const git_oid *oid;
-
-    oid = git_object_id(self->obj);
-    assert(oid);
-
-    return git_oid_to_python(oid);
+    return git_oid_to_python(Object__id(self));
 }
 
 PyDoc_STRVAR(Object_oid__doc__,
@@ -83,12 +108,7 @@ PyDoc_STRVAR(Object_hex__doc__,
 PyObject *
 Object_hex__get__(Object *self)
 {
-    const git_oid *oid;
-
-    oid = git_object_id(self->obj);
-    assert(oid);
-
-    return git_oid_to_py_str(oid);
+    return git_oid_to_py_str(Object__id(self));
 }
 
 
@@ -98,16 +118,15 @@ PyDoc_STRVAR(Object_short_id__doc__,
 PyObject *
 Object_short_id__get__(Object *self)
 {
+    if (Object__load(self) == NULL) { return NULL; } // Lazy load
+
     git_buf short_id = { NULL, 0, 0 };
-    PyObject *py_short_id;
-
     int err = git_object_short_id(&short_id, self->obj);
-
     if (err != GIT_OK)
         return Error_set(err);
 
-    py_short_id = to_unicode_n(short_id.ptr, short_id.size, NULL, "strict");
-    git_buf_free(&short_id);
+    PyObject *py_short_id = to_unicode_n(short_id.ptr, short_id.size, NULL, "strict");
+    git_buf_dispose(&short_id);
     return py_short_id;
 }
 
@@ -119,7 +138,16 @@ PyDoc_STRVAR(Object_type__doc__,
 PyObject *
 Object_type__get__(Object *self)
 {
-    return PyInt_FromLong(git_object_type(self->obj));
+    return PyLong_FromLong(Object__type(self));
+}
+
+PyDoc_STRVAR(Object_type_str__doc__,
+    "One of the 'commit', 'tree', 'blob' or 'tag' strings.");
+
+PyObject *
+Object_type_str__get__(Object *self)
+{
+    return to_path(git_object_type2string(Object__type(self)));
 }
 
 PyDoc_STRVAR(Object__pointer__doc__, "Get the object's pointer. For internal use only.");
@@ -127,9 +155,40 @@ PyObject *
 Object__pointer__get__(Object *self)
 {
     /* Bytes means a raw buffer */
+    if (Object__load(self) == NULL) { return NULL; } // Lazy load
     return PyBytes_FromStringAndSize((char *) &self->obj, sizeof(git_object *));
 }
 
+PyDoc_STRVAR(Object_name__doc__,
+             "Name (will be None if the object was not reached trough a tree)");
+PyObject *
+Object_name__get__(Object *self)
+{
+    if (self->entry == NULL)
+        Py_RETURN_NONE;
+
+    return to_path(git_tree_entry_name(self->entry));
+}
+
+PyDoc_STRVAR(Object_raw_name__doc__, "Name (bytes).");
+
+PyObject *
+Object_raw_name__get__(Object *self)
+{
+    return PyBytes_FromString(git_tree_entry_name(self->entry));
+}
+
+PyDoc_STRVAR(Object_filemode__doc__,
+             "Filemode (will be None if the object was not reached trough a tree)");
+PyObject *
+Object_filemode__get__(Object *self)
+{
+    if (self->entry == NULL)
+        Py_RETURN_NONE;
+
+    return PyLong_FromLong(git_tree_entry_filemode(self->entry));
+}
+
 
 PyDoc_STRVAR(Object_read_raw__doc__,
   "read_raw()\n"
@@ -139,13 +198,17 @@ PyDoc_STRVAR(Object_read_raw__doc__,
 PyObject *
 Object_read_raw(Object *self)
 {
-    const git_oid *oid;
-    git_odb_object *obj;
+    int err;
+    git_odb *odb;
     PyObject *aux;
 
-    oid = git_object_id(self->obj);
+    err = git_repository_odb(&odb, self->repo->repo);
+    if (err < 0)
+        return Error_set(err);
 
-    obj = Repository_read_raw(self->repo->repo, oid, GIT_OID_HEXSZ);
+    const git_oid *oid = Object__id(self);
+    git_odb_object *obj = Odb_read_raw(odb, oid, GIT_OID_HEXSZ);
+    git_odb_free(odb);
     if (obj == NULL)
         return NULL;
 
@@ -169,6 +232,8 @@ Object_peel(Object *self, PyObject *py_type)
     git_otype otype;
     git_object *peeled;
 
+    if (Object__load(self) == NULL) { return NULL; } // Lazy load
+
     otype = py_object_to_otype(py_type);
     if (otype == GIT_OBJ_BAD)
         return NULL;
@@ -177,13 +242,13 @@ Object_peel(Object *self, PyObject *py_type)
     if (err < 0)
         return Error_set(err);
 
-    return wrap_object(peeled, self->repo);
+    return wrap_object(peeled, self->repo, NULL);
 }
 
 Py_hash_t
-Object_hash(Object *object)
+Object_hash(Object *self)
 {
-    const git_oid *oid = git_object_id(object->obj);
+    const git_oid *oid = Object__id(self);
     PyObject *py_oid = git_oid_to_py_str(oid);
     Py_hash_t ret = PyObject_Hash(py_oid);
     Py_DECREF(py_oid);
@@ -194,32 +259,19 @@ PyObject *
 Object_richcompare(PyObject *o1, PyObject *o2, int op)
 {
     PyObject *res;
-    Object *obj1;
-    Object *obj2;
 
     if (!PyObject_TypeCheck(o2, &ObjectType)) {
         Py_INCREF(Py_NotImplemented);
         return Py_NotImplemented;
     }
 
+    int equal = git_oid_equal(Object__id((Object *)o1), Object__id((Object *)o2));
     switch (op) {
         case Py_NE:
-            obj1 = (Object *) o1;
-            obj2 = (Object *) o2;
-            if (git_oid_equal(git_object_id(obj1->obj), git_object_id(obj2->obj))) {
-                res = Py_False;
-            } else {
-                res = Py_True;
-            }
+            res = (equal) ? Py_False : Py_True;
             break;
         case Py_EQ:
-            obj1 = (Object *) o1;
-            obj2 = (Object *) o2;
-            if (git_oid_equal(git_object_id(obj1->obj), git_object_id(obj2->obj))) {
-                res = Py_True;
-            } else {
-                res = Py_False;
-            }
+            res = (equal) ? Py_True : Py_False;
             break;
         case Py_LT:
         case Py_LE:
@@ -242,7 +294,12 @@ PyGetSetDef Object_getseters[] = {
     GETTER(Object, hex),
     GETTER(Object, short_id),
     GETTER(Object, type),
+    GETTER(Object, type_str),
     GETTER(Object, _pointer),
+    // These come from git_tree_entry
+    GETTER(Object, name),
+    GETTER(Object, raw_name),
+    GETTER(Object, filemode),
     {NULL}
 };
 
@@ -279,7 +336,7 @@ PyTypeObject ObjectType = {
     Object__doc__,                             /* tp_doc            */
     0,                                         /* tp_traverse       */
     0,                                         /* tp_clear          */
-    Object_richcompare,                        /* tp_richcompare    */
+    (richcmpfunc)Object_richcompare,           /* tp_richcompare    */
     0,                                         /* tp_weaklistoffset */
     0,                                         /* tp_iter           */
     0,                                         /* tp_iternext       */
@@ -297,11 +354,13 @@ PyTypeObject ObjectType = {
 };
 
 PyObject *
-wrap_object(git_object *c_object, Repository *repo)
+wrap_object(git_object *c_object, Repository *repo, const git_tree_entry *entry)
 {
     Object *py_obj = NULL;
 
-    switch (git_object_type(c_object)) {
+    git_object_t obj_type = (c_object) ? git_object_type(c_object) : git_tree_entry_type(entry);
+
+    switch (obj_type) {
         case GIT_OBJ_COMMIT:
             py_obj = PyObject_New(Object, &CommitType);
             break;
@@ -324,6 +383,7 @@ wrap_object(git_object *c_object, Repository *repo)
             py_obj->repo = repo;
             Py_INCREF(repo);
         }
+        py_obj->entry = entry;
     }
     return (PyObject *)py_obj;
 }
