@@ -46,7 +46,7 @@ get_search_path(long level)
         return Error_set(err);
 
     py_path = to_unicode(buf.ptr, NULL, NULL);
-    git_buf_free(&buf);
+    git_buf_dispose(&buf);
 
     if (!py_path)
         return NULL;
@@ -65,13 +65,82 @@ option(PyObject *self, PyObject *args)
     if (!py_option)
         return NULL;
 
-    if (!PyInt_Check(py_option))
+    if (!PyLong_Check(py_option))
         return Error_type_error(
             "option should be an integer, got %.200s", py_option);
 
-    option = PyInt_AsLong(py_option);
+    option = PyLong_AsLong(py_option);
 
     switch (option) {
+
+        case GIT_OPT_GET_MWINDOW_SIZE:
+        {
+            size_t size;
+
+            error = git_libgit2_opts(GIT_OPT_GET_MWINDOW_SIZE, &size);
+            if (error < 0)
+                return Error_set(error);
+
+            return PyLong_FromSize_t(size);
+        }
+
+        case GIT_OPT_SET_MWINDOW_SIZE:
+        {
+            size_t size;
+            PyObject *py_size;
+
+            py_size = PyTuple_GetItem(args, 1);
+            if (!py_size)
+                return NULL;
+
+            if (!PyLong_Check(py_size))
+                return Error_type_error(
+                    "size should be an integer, got %.200s", py_size);
+
+            size = PyLong_AsSize_t(py_size);
+            error = git_libgit2_opts(GIT_OPT_SET_MWINDOW_SIZE, size);
+            if (error  < 0)
+                return Error_set(error);
+
+            Py_RETURN_NONE;
+        }
+
+        case GIT_OPT_GET_MWINDOW_MAPPED_LIMIT:
+        {
+            size_t limit;
+
+            error = git_libgit2_opts(GIT_OPT_GET_MWINDOW_MAPPED_LIMIT, &limit);
+            if (error < 0)
+                return Error_set(error);
+
+            return PyLong_FromSize_t(limit);
+        }
+
+        case GIT_OPT_SET_MWINDOW_MAPPED_LIMIT:
+        {
+            size_t limit;
+            PyObject *py_limit;
+
+            py_limit = PyTuple_GetItem(args, 1);
+            if (!py_limit)
+                return NULL;
+
+            if (PyLong_Check(py_limit)) {
+                limit = PyLong_AsSize_t(py_limit);
+            } else if (PyLong_Check(py_limit)) {
+                limit = PyLong_AsSize_t(py_limit);
+            } else {
+                return Error_type_error(
+                    "limit should be an integer, got %.200s", py_limit);
+            }
+
+            error = git_libgit2_opts(GIT_OPT_SET_MWINDOW_MAPPED_LIMIT, limit);
+            if (error < 0)
+                return Error_set(error);
+
+            Py_RETURN_NONE;
+        }
+
         case GIT_OPT_GET_SEARCH_PATH:
         {
             PyObject *py_level;
@@ -80,11 +149,11 @@ option(PyObject *self, PyObject *args)
             if (!py_level)
                 return NULL;
 
-            if (!PyInt_Check(py_level))
+            if (!PyLong_Check(py_level))
                 return Error_type_error(
                     "level should be an integer, got %.200s", py_level);
 
-            return get_search_path(PyInt_AsLong(py_level));
+            return get_search_path(PyLong_AsLong(py_level));
         }
 
         case GIT_OPT_SET_SEARCH_PATH:
@@ -101,7 +170,7 @@ option(PyObject *self, PyObject *args)
             if (!py_path)
                 return NULL;
 
-            if (!PyInt_Check(py_level))
+            if (!PyLong_Check(py_level))
                 return Error_type_error(
                     "level should be an integer, got %.200s", py_level);
 
@@ -110,7 +179,7 @@ option(PyObject *self, PyObject *args)
                 return NULL;
 
             err = git_libgit2_opts(
-                GIT_OPT_SET_SEARCH_PATH, PyInt_AsLong(py_level), path);
+                GIT_OPT_SET_SEARCH_PATH, PyLong_AsLong(py_level), path);
             Py_DECREF(tpath);
 
             if (err < 0)
@@ -119,74 +188,6 @@ option(PyObject *self, PyObject *args)
             Py_RETURN_NONE;
         }
 
-        case GIT_OPT_GET_MWINDOW_SIZE:
-        {
-            size_t size;
-
-            error = git_libgit2_opts(GIT_OPT_GET_MWINDOW_SIZE, &size);
-            if (error < 0)
-                return Error_set(error);
-
-            return PyInt_FromSize_t(size);
-        }
-
-        case GIT_OPT_SET_MWINDOW_SIZE:
-        {
-            size_t size;
-            PyObject *py_size;
-
-            py_size = PyTuple_GetItem(args, 1);
-            if (!py_size)
-                return NULL;
-
-            if (!PyInt_Check(py_size))
-                return Error_type_error(
-                    "size should be an integer, got %.200s", py_size);
-
-            size = PyInt_AsSize_t(py_size);
-            error = git_libgit2_opts(GIT_OPT_SET_MWINDOW_SIZE, size);
-            if (error  < 0)
-                return Error_set(error);
-
-            Py_RETURN_NONE;
-        }
-
-        case GIT_OPT_GET_MWINDOW_MAPPED_LIMIT:
-        {
-            size_t limit;
-
-            error = git_libgit2_opts(GIT_OPT_GET_MWINDOW_MAPPED_LIMIT, &limit);
-            if (error < 0)
-                return Error_set(error);
-
-            return PyInt_FromSize_t(limit);
-        }
-
-        case GIT_OPT_SET_MWINDOW_MAPPED_LIMIT:
-        {
-            size_t limit;
-            PyObject *py_limit;
-
-            py_limit = PyTuple_GetItem(args, 1);
-            if (!py_limit)
-                return NULL;
-
-            if (PyInt_Check(py_limit)) {
-                limit = PyInt_AsSize_t(py_limit);
-            } else if (PyLong_Check(py_limit)) {
-                limit = PyLong_AsSize_t(py_limit);
-            } else {
-                return Error_type_error(
-                    "limit should be an integer, got %.200s", py_limit);
-            }
-
-            error = git_libgit2_opts(GIT_OPT_SET_MWINDOW_MAPPED_LIMIT, limit);
-            if (error < 0)
-                return Error_set(error);
-
-            Py_RETURN_NONE;
-        }
-
         case GIT_OPT_SET_CACHE_OBJECT_LIMIT:
         {
             size_t limit;
@@ -201,12 +202,12 @@ option(PyObject *self, PyObject *args)
             if (!py_limit)
                 return NULL;
 
-            if (!PyInt_Check(py_limit))
+            if (!PyLong_Check(py_limit))
                 return Error_type_error(
                     "limit should be an integer, got %.200s", py_limit);
 
-            object_type = PyInt_AsLong(py_object_type);
-            limit = PyInt_AsSize_t(py_limit);
+            object_type = PyLong_AsLong(py_object_type);
+            limit = PyLong_AsSize_t(py_limit);
             error = git_libgit2_opts(
                 GIT_OPT_SET_CACHE_OBJECT_LIMIT, object_type, limit);
 
@@ -225,11 +226,11 @@ option(PyObject *self, PyObject *args)
             if (!py_max_size)
                 return NULL;
 
-            if (!PyInt_Check(py_max_size))
+            if (!PyLong_Check(py_max_size))
                 return Error_type_error(
                     "max_size should be an integer, got %.200s", py_max_size);
 
-            max_size = PyInt_AsSize_t(py_max_size);
+            max_size = PyLong_AsSize_t(py_max_size);
             error = git_libgit2_opts(GIT_OPT_SET_CACHE_MAX_SIZE, max_size);
             if (error < 0)
                 return Error_set(error);
@@ -237,25 +238,6 @@ option(PyObject *self, PyObject *args)
             Py_RETURN_NONE;
         }
 
-        case GIT_OPT_ENABLE_CACHING:
-        {
-            int flag;
-            PyObject *py_flag;
-
-            py_flag = PyTuple_GetItem(args, 1);
-
-            if (!PyInt_Check(py_flag))
-                return Error_type_error(
-                    "flag should be an integer, got %.200s", py_flag);
-
-            flag = PyInt_AsSize_t(py_flag);
-            error = git_libgit2_opts(GIT_OPT_ENABLE_CACHING, flag);
-            if (error < 0)
-                return Error_set(error);
-
-            Py_RETURN_NONE;
-        }
-
         case GIT_OPT_GET_CACHED_MEMORY:
         {
             size_t current;
@@ -266,12 +248,85 @@ option(PyObject *self, PyObject *args)
             if (error < 0)
                 return Error_set(error);
 
-            PyTuple_SetItem(tup, 0, PyInt_FromLong(current));
-            PyTuple_SetItem(tup, 1, PyInt_FromLong(allowed));
+            PyTuple_SetItem(tup, 0, PyLong_FromLong(current));
+            PyTuple_SetItem(tup, 1, PyLong_FromLong(allowed));
 
             return tup;
         }
 
+        case GIT_OPT_SET_SSL_CERT_LOCATIONS:
+        {
+            PyObject *py_file, *py_dir;
+            const char *file_path, *dir_path;
+            int err;
+
+            py_file = PyTuple_GetItem(args, 1);
+            py_dir = PyTuple_GetItem(args, 2);
+
+            /* py_file and py_dir are only valid if they are strings */
+            if (PyUnicode_Check(py_file) || PyBytes_Check(py_file)) {
+                file_path = py_str_to_c_str(py_file, Py_FileSystemDefaultEncoding);
+            } else {
+                file_path = NULL;
+            }
+
+            if (PyUnicode_Check(py_dir) || PyBytes_Check(py_dir)) {
+                dir_path = py_str_to_c_str(py_dir, Py_FileSystemDefaultEncoding);
+            } else {
+                dir_path = NULL;
+            }
+
+            err = git_libgit2_opts(GIT_OPT_SET_SSL_CERT_LOCATIONS, file_path, dir_path);
+
+            if (err < 0)
+                return Error_set(err);
+
+            Py_RETURN_NONE;
+        }
+
+        // int enabled
+        case GIT_OPT_ENABLE_CACHING:
+        case GIT_OPT_ENABLE_STRICT_OBJECT_CREATION:
+        case GIT_OPT_ENABLE_STRICT_SYMBOLIC_REF_CREATION:
+        case GIT_OPT_ENABLE_OFS_DELTA:
+        case GIT_OPT_ENABLE_FSYNC_GITDIR:
+        case GIT_OPT_ENABLE_STRICT_HASH_VERIFICATION:
+        case GIT_OPT_ENABLE_UNSAVED_INDEX_SAFETY:
+        {
+            PyObject *py_enabled;
+            int enabled;
+
+            py_enabled = PyTuple_GetItem(args, 1);
+            if (!py_enabled)
+                return NULL;
+
+            if (!PyLong_Check(py_enabled))
+                return Error_type_error("expected integer, got %.200s", py_enabled);
+
+            enabled = PyLong_AsSize_t(py_enabled);
+            error = git_libgit2_opts(option, enabled);
+            if (error < 0)
+                return Error_set(error);
+
+            Py_RETURN_NONE;
+        }
+
+        // Not implemented
+        case GIT_OPT_GET_TEMPLATE_PATH:
+        case GIT_OPT_SET_TEMPLATE_PATH:
+        case GIT_OPT_SET_USER_AGENT:
+        case GIT_OPT_SET_SSL_CIPHERS:
+        case GIT_OPT_GET_USER_AGENT:
+        case GIT_OPT_GET_WINDOWS_SHAREMODE:
+        case GIT_OPT_SET_WINDOWS_SHAREMODE:
+        case GIT_OPT_SET_ALLOCATOR:
+        case GIT_OPT_GET_PACK_MAX_OBJECTS:
+        case GIT_OPT_SET_PACK_MAX_OBJECTS:
+        {
+            Py_INCREF(Py_NotImplemented);
+            return Py_NotImplemented;
+        }
+
     }
 
     PyErr_SetString(PyExc_ValueError, "unknown/unsupported option value");
