@@ -41,6 +41,10 @@ extern PyObject *AlreadyExistsError;
 extern PyObject *InvalidSpecError;
 
 extern PyTypeObject RepositoryType;
+extern PyTypeObject OdbType;
+extern PyTypeObject OdbBackendType;
+extern PyTypeObject OdbBackendPackType;
+extern PyTypeObject OdbBackendLooseType;
 extern PyTypeObject OidType;
 extern PyTypeObject ObjectType;
 extern PyTypeObject CommitType;
@@ -55,7 +59,6 @@ extern PyTypeObject DiffStatsType;
 extern PyTypeObject PatchType;
 extern PyTypeObject TreeType;
 extern PyTypeObject TreeBuilderType;
-extern PyTypeObject TreeEntryType;
 extern PyTypeObject TreeIterType;
 extern PyTypeObject BlobType;
 extern PyTypeObject TagType;
@@ -70,6 +73,7 @@ extern PyTypeObject RefspecType;
 extern PyTypeObject NoteType;
 extern PyTypeObject NoteIterType;
 extern PyTypeObject WorktreeType;
+extern PyTypeObject MailmapType;
 
 
 PyDoc_STRVAR(discover_repository__doc__,
@@ -98,7 +102,7 @@ discover_repository(PyObject *self, PyObject *args)
         return Error_set_str(err, path);
 
     py_repo_path = to_path(repo_path.ptr);
-    git_buf_free(&repo_path);
+    git_buf_dispose(&repo_path);
 
     return py_repo_path;
 };
@@ -206,15 +210,43 @@ reference_is_valid_name(PyObject *self, PyObject *py_refname)
 }
 
 
+PyDoc_STRVAR(tree_entry_cmp__doc__,
+    "tree_entry_obj(a, b) -> int\n"
+    "\n"
+    "Rich comparison for objects, only available when the objects have been\n"
+    "obtained through a tree. The sort criteria is the one Git uses to sort\n"
+    "tree entries in a tree object. This function wraps git_tree_entry_cmp.\n"
+    "\n"
+    "Returns < 0 if a is before b, > 0 if a is after b, and 0 if a and b are\n"
+    "the same.");
+
+PyObject *
+tree_entry_cmp(PyObject *self, PyObject *args)
+{
+    Object *a, *b;
+    int cmp;
+
+    if (!PyArg_ParseTuple(args, "O!O!", &ObjectType, &a, &ObjectType, &b))
+        return NULL;
+
+    if (a->entry == NULL || b->entry == NULL) {
+        PyErr_SetString(PyExc_ValueError, "objects lack entry information");
+        return NULL;
+    }
+
+    cmp = git_tree_entry_cmp(a->entry, b->entry);
+    return PyLong_FromLong(cmp);
+}
+
+
 PyMethodDef module_methods[] = {
-    {"init_file_backend", init_file_backend, METH_VARARGS,
-    init_file_backend__doc__},
-    {"discover_repository", discover_repository, METH_VARARGS,
-     discover_repository__doc__},
-    {"hashfile", hashfile, METH_VARARGS, hashfile__doc__},
+    {"discover_repository", discover_repository, METH_VARARGS, discover_repository__doc__},
     {"hash", hash, METH_VARARGS, hash__doc__},
-    {"reference_is_valid_name", reference_is_valid_name, METH_O, reference_is_valid_name__doc__},
+    {"hashfile", hashfile, METH_VARARGS, hashfile__doc__},
+    {"init_file_backend", init_file_backend, METH_VARARGS, init_file_backend__doc__},
     {"option", option, METH_VARARGS, option__doc__},
+    {"reference_is_valid_name", reference_is_valid_name, METH_O, reference_is_valid_name__doc__},
+    {"tree_entry_cmp", tree_entry_cmp, METH_VARARGS, tree_entry_cmp__doc__},
     {NULL}
 };
 
@@ -231,16 +263,33 @@ moduleinit(PyObject* m)
     ADD_CONSTANT_STR(m, LIBGIT2_VERSION)
 
     /* libgit2 options */
-    ADD_CONSTANT_INT(m, GIT_OPT_GET_SEARCH_PATH);
-    ADD_CONSTANT_INT(m, GIT_OPT_SET_SEARCH_PATH);
     ADD_CONSTANT_INT(m, GIT_OPT_GET_MWINDOW_SIZE);
     ADD_CONSTANT_INT(m, GIT_OPT_SET_MWINDOW_SIZE);
     ADD_CONSTANT_INT(m, GIT_OPT_GET_MWINDOW_MAPPED_LIMIT);
     ADD_CONSTANT_INT(m, GIT_OPT_SET_MWINDOW_MAPPED_LIMIT);
+    ADD_CONSTANT_INT(m, GIT_OPT_GET_SEARCH_PATH);
+    ADD_CONSTANT_INT(m, GIT_OPT_SET_SEARCH_PATH);
     ADD_CONSTANT_INT(m, GIT_OPT_SET_CACHE_OBJECT_LIMIT);
-    ADD_CONSTANT_INT(m, GIT_OPT_GET_CACHED_MEMORY);
-    ADD_CONSTANT_INT(m, GIT_OPT_ENABLE_CACHING);
     ADD_CONSTANT_INT(m, GIT_OPT_SET_CACHE_MAX_SIZE);
+    ADD_CONSTANT_INT(m, GIT_OPT_ENABLE_CACHING);
+    ADD_CONSTANT_INT(m, GIT_OPT_GET_CACHED_MEMORY);
+    ADD_CONSTANT_INT(m, GIT_OPT_GET_TEMPLATE_PATH);
+    ADD_CONSTANT_INT(m, GIT_OPT_SET_TEMPLATE_PATH);
+    ADD_CONSTANT_INT(m, GIT_OPT_SET_SSL_CERT_LOCATIONS);
+    ADD_CONSTANT_INT(m, GIT_OPT_SET_USER_AGENT);
+    ADD_CONSTANT_INT(m, GIT_OPT_ENABLE_STRICT_OBJECT_CREATION);
+    ADD_CONSTANT_INT(m, GIT_OPT_ENABLE_STRICT_SYMBOLIC_REF_CREATION);
+    ADD_CONSTANT_INT(m, GIT_OPT_SET_SSL_CIPHERS);
+    ADD_CONSTANT_INT(m, GIT_OPT_GET_USER_AGENT);
+    ADD_CONSTANT_INT(m, GIT_OPT_ENABLE_OFS_DELTA);
+    ADD_CONSTANT_INT(m, GIT_OPT_ENABLE_FSYNC_GITDIR);
+    ADD_CONSTANT_INT(m, GIT_OPT_GET_WINDOWS_SHAREMODE);
+    ADD_CONSTANT_INT(m, GIT_OPT_SET_WINDOWS_SHAREMODE);
+    ADD_CONSTANT_INT(m, GIT_OPT_ENABLE_STRICT_HASH_VERIFICATION);
+    ADD_CONSTANT_INT(m, GIT_OPT_SET_ALLOCATOR);
+    ADD_CONSTANT_INT(m, GIT_OPT_ENABLE_UNSAVED_INDEX_SAFETY);
+    ADD_CONSTANT_INT(m, GIT_OPT_GET_PACK_MAX_OBJECTS);
+    ADD_CONSTANT_INT(m, GIT_OPT_SET_PACK_MAX_OBJECTS);
 
     /* Errors */
     GitError = PyErr_NewException("_pygit2.GitError", NULL, NULL);
@@ -259,6 +308,17 @@ moduleinit(PyObject* m)
     INIT_TYPE(RepositoryType, NULL, PyType_GenericNew)
     ADD_TYPE(m, Repository)
 
+    /* Odb */
+    INIT_TYPE(OdbType, NULL, PyType_GenericNew)
+    ADD_TYPE(m, Odb)
+
+    INIT_TYPE(OdbBackendType, NULL, NULL)
+    ADD_TYPE(m, OdbBackend)
+    INIT_TYPE(OdbBackendPackType, &OdbBackendType, PyType_GenericNew)
+    ADD_TYPE(m, OdbBackendPack)
+    INIT_TYPE(OdbBackendLooseType, &OdbBackendType, PyType_GenericNew)
+    ADD_TYPE(m, OdbBackendLoose)
+
     /* Oid */
     INIT_TYPE(OidType, NULL, PyType_GenericNew)
     ADD_TYPE(m, Oid)
@@ -274,7 +334,6 @@ moduleinit(PyObject* m)
     INIT_TYPE(CommitType, &ObjectType, NULL)
     INIT_TYPE(SignatureType, NULL, PyType_GenericNew)
     INIT_TYPE(TreeType, &ObjectType, NULL)
-    INIT_TYPE(TreeEntryType, NULL, NULL)
     INIT_TYPE(TreeIterType, NULL, NULL)
     INIT_TYPE(TreeBuilderType, NULL, NULL)
     INIT_TYPE(BlobType, &ObjectType, NULL)
@@ -283,7 +342,6 @@ moduleinit(PyObject* m)
     ADD_TYPE(m, Commit)
     ADD_TYPE(m, Signature)
     ADD_TYPE(m, Tree)
-    ADD_TYPE(m, TreeEntry)
     ADD_TYPE(m, TreeBuilder)
     ADD_TYPE(m, Blob)
     ADD_TYPE(m, Tag)
@@ -486,6 +544,10 @@ moduleinit(PyObject* m)
     ADD_CONSTANT_INT(m, GIT_STASH_APPLY_DEFAULT);
     ADD_CONSTANT_INT(m, GIT_STASH_APPLY_REINSTATE_INDEX);
 
+    /* Mailmap */
+    INIT_TYPE(MailmapType, NULL, PyType_GenericNew)
+    ADD_TYPE(m, Mailmap)
+
     /* Global initialization of libgit2 */
     git_libgit2_init();
 
@@ -493,33 +555,22 @@ moduleinit(PyObject* m)
 }
 
 
-#if PY_MAJOR_VERSION < 3
-    PyMODINIT_FUNC
-    init_pygit2(void)
-    {
-        PyObject* m;
-        m = Py_InitModule3("_pygit2", module_methods,
-                           "Python bindings for libgit2.");
-        moduleinit(m);
-    }
-#else
-    struct PyModuleDef moduledef = {
-        PyModuleDef_HEAD_INIT,
-        "_pygit2",                       /* m_name */
-        "Python bindings for libgit2.",  /* m_doc */
-        -1,                              /* m_size */
-        module_methods,                  /* m_methods */
-        NULL,                            /* m_reload */
-        NULL,                            /* m_traverse */
-        NULL,                            /* m_clear */
-        NULL,                            /* m_free */
-    };
+struct PyModuleDef moduledef = {
+    PyModuleDef_HEAD_INIT,
+    "_pygit2",                       /* m_name */
+    "Python bindings for libgit2.",  /* m_doc */
+    -1,                              /* m_size */
+    module_methods,                  /* m_methods */
+    NULL,                            /* m_reload */
+    NULL,                            /* m_traverse */
+    NULL,                            /* m_clear */
+    NULL,                            /* m_free */
+};
 
-    PyMODINIT_FUNC
-    PyInit__pygit2(void)
-    {
-        PyObject* m;
-        m = PyModule_Create(&moduledef);
-        return moduleinit(m);
-    }
-#endif
+PyMODINIT_FUNC
+PyInit__pygit2(void)
+{
+    PyObject* m;
+    m = PyModule_Create(&moduledef);
+    return moduleinit(m);
+}
