@@ -54,16 +54,20 @@ PyObject *
 RefLogIter_iternext(RefLogIter *self)
 {
     const git_reflog_entry *entry;
+    const char * entry_message;
     RefLogEntry *py_entry;
     int err;
 
     if (self->i < self->size) {
         entry = git_reflog_entry_byindex(self->reflog, self->i);
         py_entry = PyObject_New(RefLogEntry, &RefLogEntryType);
+        if (py_entry == NULL)
+            return NULL;
 
         py_entry->oid_old = git_oid_to_python(git_reflog_entry_id_old(entry));
         py_entry->oid_new = git_oid_to_python(git_reflog_entry_id_new(entry));
-        py_entry->message = strdup(git_reflog_entry_message(entry));
+        entry_message = git_reflog_entry_message(entry);
+        py_entry->message = (entry_message != NULL) ? strdup(entry_message) : NULL;
         err = git_signature_dup(&py_entry->signature,
                                 git_reflog_entry_committer(entry));
         if (err < 0)
@@ -342,11 +346,11 @@ PyDoc_STRVAR(Reference_type__doc__,
 PyObject *
 Reference_type__get__(Reference *self)
 {
-    git_ref_t c_type;
+    git_reference_t c_type;
 
     CHECK_REFERENCE(self);
     c_type = git_reference_type(self->reference);
-    return PyInt_FromLong(c_type);
+    return PyLong_FromLong(c_type);
 }
 
 
@@ -405,7 +409,7 @@ Reference_peel(Reference *self, PyObject *args)
     if (err < 0)
         return Error_set(err);
 
-    return wrap_object(obj, self->repo);
+    return wrap_object(obj, self->repo, NULL);
 }
 
 PyObject *
