@@ -31,6 +31,7 @@
 #include "types.h"
 #include "reference.h"
 #include "utils.h"
+#include "odb.h"
 #include "object.h"
 #include "oid.h"
 #include "note.h"
@@ -61,18 +62,6 @@ extern PyTypeObject NoteIterType;
 /* forward-declaration for Repsository._from_c() */
 PyTypeObject RepositoryType;
 
-git_otype
-int_to_loose_object_type(int type_id)
-{
-    switch((git_otype)type_id) {
-        case GIT_OBJ_COMMIT: return GIT_OBJ_COMMIT;
-        case GIT_OBJ_TREE: return GIT_OBJ_TREE;
-        case GIT_OBJ_BLOB: return GIT_OBJ_BLOB;
-        case GIT_OBJ_TAG: return GIT_OBJ_TAG;
-        default: return GIT_OBJ_BAD;
-    }
-}
-
 PyObject *
 wrap_repository(git_repository *c_repo)
 {
@@ -182,42 +171,6 @@ Repository_clear(Repository *self)
     return 0;
 }
 
-static int
-Repository_build_as_iter(const git_oid *oid, void *accum)
-{
-    int err;
-    PyObject *py_oid = git_oid_to_python(oid);
-
-    err = PyList_Append((PyObject*)accum, py_oid);
-    Py_DECREF(py_oid);
-    return err;
-}
-
-PyObject *
-Repository_as_iter(Repository *self)
-{
-    git_odb *odb;
-    int err;
-    PyObject *accum = PyList_New(0);
-    PyObject *ret;
-
-    err = git_repository_odb(&odb, self->repo);
-    if (err < 0)
-        return Error_set(err);
-
-    err = git_odb_foreach(odb, Repository_build_as_iter, (void*)accum);
-    git_odb_free(odb);
-    if (err == GIT_EUSER)
-        return NULL;
-    if (err < 0)
-        return Error_set(err);
-
-    ret = PyObject_GetIter(accum);
-    Py_DECREF(accum);
-
-    return ret;
-}
-
 
 PyDoc_STRVAR(Repository_head__doc__,
   "Current head reference of the repository.");
@@ -314,7 +267,7 @@ Repository_git_object_lookup_prefix(Repository *self, PyObject *key)
 
     err = git_object_lookup_prefix(&obj, self->repo, &oid, len, GIT_OBJ_ANY);
     if (err == 0)
-        return wrap_object(obj, self);
+        return wrap_object(obj, self, NULL);
 
     if (err == GIT_ENOTFOUND)
         Py_RETURN_NONE;
@@ -394,125 +347,17 @@ Repository_revparse_single(Repository *self, PyObject *py_spec)
 
     /* 2- Lookup */
     err = git_revparse_single(&c_obj, self->repo, c_spec);
-
     if (err < 0) {
         PyObject *err_obj = Error_set_str(err, c_spec);
         Py_DECREF(tspec);
         return err_obj;
     }
+
     Py_DECREF(tspec);
 
-    return wrap_object(c_obj, self);
+    return wrap_object(c_obj, self, NULL);
 }
 
-git_odb_object *
-Repository_read_raw(git_repository *repo, const git_oid *oid, size_t len)
-{
-    git_odb *odb;
-    git_odb_object *obj;
-    int err;
-
-    err = git_repository_odb(&odb, repo);
-    if (err < 0) {
-        Error_set(err);
-        return NULL;
-    }
-
-    err = git_odb_read_prefix(&obj, odb, oid, (unsigned int)len);
-    git_odb_free(odb);
-    if (err < 0) {
-        Error_set_oid(err, oid, len);
-        return NULL;
-    }
-
-    return obj;
-}
-
-
-PyDoc_STRVAR(Repository_read__doc__,
-  "read(oid) -> type, data, size\n"
-  "\n"
-  "Read raw object data from the repository.");
-
-PyObject *
-Repository_read(Repository *self, PyObject *py_hex)
-{
-    git_oid oid;
-    git_odb_object *obj;
-    size_t len;
-    PyObject* tuple;
-
-    len = py_oid_to_git_oid(py_hex, &oid);
-    if (len == 0)
-        return NULL;
-
-    obj = Repository_read_raw(self->repo, &oid, len);
-    if (obj == NULL)
-        return NULL;
-
-    tuple = Py_BuildValue(
-    #if PY_MAJOR_VERSION == 2
-        "(ns#)",
-    #else
-        "(ny#)",
-    #endif
-        git_odb_object_type(obj),
-        git_odb_object_data(obj),
-        git_odb_object_size(obj));
-
-    git_odb_object_free(obj);
-    return tuple;
-}
-
-
-PyDoc_STRVAR(Repository_write__doc__,
-    "write(type, data) -> Oid\n"
-    "\n"
-    "Write raw object data into the repository. First arg is the object\n"
-    "type, the second one a buffer with data. Return the Oid of the created\n"
-    "object.");
-
-PyObject *
-Repository_write(Repository *self, PyObject *args)
-{
-    int err;
-    git_oid oid;
-    git_odb *odb;
-    git_odb_stream* stream;
-    int type_id;
-    const char* buffer;
-    Py_ssize_t buflen;
-    git_otype type;
-
-    if (!PyArg_ParseTuple(args, "Is#", &type_id, &buffer, &buflen))
-        return NULL;
-
-    type = int_to_loose_object_type(type_id);
-    if (type == GIT_OBJ_BAD)
-        return PyErr_Format(PyExc_ValueError, "%d", type_id);
-
-    err = git_repository_odb(&odb, self->repo);
-    if (err < 0)
-        return Error_set(err);
-
-    err = git_odb_open_wstream(&stream, odb, buflen, type);
-    git_odb_free(odb);
-    if (err < 0)
-        return Error_set(err);
-
-    err = git_odb_stream_write(stream, buffer, buflen);
-    if (err) {
-        git_odb_stream_free(stream);
-        return Error_set(err);
-    }
-
-    err = git_odb_stream_finalize_write(&oid, stream);
-    git_odb_stream_free(stream);
-    if (err)
-        return Error_set(err);
-
-    return git_oid_to_python(&oid);
-}
 
 PyDoc_STRVAR(Repository_path__doc__,
   "The normalized path to the git repository.");
@@ -637,40 +482,69 @@ Repository_merge_base(Repository *self, PyObject *args)
 }
 
 PyDoc_STRVAR(Repository_merge_analysis__doc__,
-  "merge_analysis(id) -> (Integer, Integer)\n"
+  "merge_analysis(their_head, our_ref='HEAD') -> (Integer, Integer)\n"
   "\n"
-  "Analyzes the given branch and determines the opportunities for merging\n"
-  "them into the HEAD of the repository\n"
+  "Analyzes the given branch and determines the opportunities for\n"
+  "merging it into a reference (defaults to HEAD).\n"
+  "\n"
+  "Parameters:\n"
+  "\n"
+  "our_ref\n"
+  "    The reference name (String) to perform the analysis from\n"
+  "\n"
+  "their_head\n"
+  "    Head (commit Oid) to merge into\n"
   "\n"
   "The first returned value is a mixture of the GIT_MERGE_ANALYSIS_NONE, _NORMAL,\n"
   "_UP_TO_DATE, _FASTFORWARD and _UNBORN flags.\n"
   "The second value is the user's preference from 'merge.ff'");
 
 PyObject *
-Repository_merge_analysis(Repository *self, PyObject *py_id)
+Repository_merge_analysis(Repository *self, PyObject *args)
 {
-    int err;
-    size_t len;
-    git_oid id;
+    char *our_ref_name = "HEAD";
+    PyObject *py_their_head;
+    PyObject *py_result = NULL;
+    git_oid head_id;
+    git_reference *our_ref;
     git_annotated_commit *commit;
     git_merge_analysis_t analysis;
     git_merge_preference_t preference;
+    int err = 0;
 
-    len = py_oid_to_git_oid(py_id, &id);
-    if (len == 0)
+    if (!PyArg_ParseTuple(args, "O|z",
+                          &py_their_head,
+                          &our_ref_name))
         return NULL;
 
-    err = git_annotated_commit_lookup(&commit, self->repo, &id);
-    if (err < 0)
-        return Error_set(err);
+    err = git_reference_lookup(&our_ref, self->repo, our_ref_name);
+    if (err < 0) {
+        PyObject *py_err = Error_set_str(err, our_ref_name);
+        return py_err;
+    }
 
-    err = git_merge_analysis(&analysis, &preference, self->repo, (const git_annotated_commit **) &commit, 1);
+    err = py_oid_to_git_oid_expand(self->repo, py_their_head, &head_id);
+    if (err < 0)
+        goto out;
+
+    err = git_annotated_commit_lookup(&commit, self->repo, &head_id);
+    if (err < 0) {
+        py_result = Error_set(err);
+        goto out;
+    }
+
+    err = git_merge_analysis_for_ref(&analysis, &preference, self->repo, our_ref, (const git_annotated_commit **) &commit, 1);
     git_annotated_commit_free(commit);
+    if (err < 0) {
+        py_result = Error_set(err);
+        goto out;
+    }
 
-    if (err < 0)
-        return Error_set(err);
+    py_result = Py_BuildValue("(ii)", analysis, preference);
 
-    return Py_BuildValue("(ii)", analysis, preference);
+out:
+    git_reference_free(our_ref);
+    return py_result;
 }
 
 PyDoc_STRVAR(Repository_merge__doc__,
@@ -804,28 +678,27 @@ Repository_walk(Repository *self, PyObject *args)
     /* Push */
     if (value != Py_None) {
         err = py_oid_to_git_oid_expand(self->repo, value, &oid);
-        if (err < 0) {
-            git_revwalk_free(walk);
-            return NULL;
-        }
+        if (err < 0)
+            goto error;
 
         err = git_revwalk_push(walk, &oid);
         if (err < 0) {
-            git_revwalk_free(walk);
-            return Error_set(err);
+            Error_set(err);
+            goto error;
         }
     }
 
     py_walker = PyObject_New(Walker, &WalkerType);
-    if (!py_walker) {
-        git_revwalk_free(walk);
-        return NULL;
+    if (py_walker) {
+        Py_INCREF(self);
+        py_walker->repo = self;
+        py_walker->walk = walk;
+        return (PyObject*)py_walker;
     }
 
-    Py_INCREF(self);
-    py_walker->repo = self;
-    py_walker->walk = walk;
-    return (PyObject*)py_walker;
+error:
+    git_revwalk_free(walk);
+    return NULL;
 }
 
 
@@ -1426,6 +1299,36 @@ Repository_lookup_reference(Repository *self, PyObject *py_name)
     return wrap_reference(c_reference, self);
 }
 
+PyDoc_STRVAR(Repository_lookup_reference_dwim__doc__,
+  "lookup_reference_dwim(name) -> Reference\n"
+  "\n"
+  "Lookup a reference by doing-what-i-mean'ing its short name.");
+
+PyObject *
+Repository_lookup_reference_dwim(Repository *self, PyObject *py_name)
+{
+    git_reference *c_reference;
+    char *c_name;
+    int err;
+
+    /* 1- Get the C name */
+    c_name = py_path_to_c_str(py_name);
+    if (c_name == NULL)
+        return NULL;
+
+    /* 2- Lookup */
+    err = git_reference_dwim(&c_reference, self->repo, c_name);
+    if (err < 0) {
+        PyObject *err_obj = Error_set_str(err, c_name);
+        free(c_name);
+        return err_obj;
+    }
+    free(c_name);
+
+    /* 3- Make an instance of Reference and return it */
+    return wrap_reference(c_reference, self);
+}
+
 PyDoc_STRVAR(Repository_create_reference_direct__doc__,
   "create_reference_direct(name, target, force)\n"
   "\n"
@@ -1503,6 +1406,31 @@ Repository_create_reference_symbolic(Repository *self,  PyObject *args,
     return wrap_reference(c_reference, self);
 }
 
+PyDoc_STRVAR(Repository_compress_references__doc__,
+  "compress_references()\n"
+  "\n"
+  "Suggest that the repository compress or optimize its references.\n"
+  "This mechanism is implementation-specific.  For on-disk reference\n"
+  "databases, for example, this may pack all loose references.");
+
+PyObject *
+Repository_compress_references(Repository *self)
+{
+    git_refdb *refdb;
+    int err;
+
+    err = git_repository_refdb(&refdb, self->repo);
+    if (err < 0)
+        return Error_set(err);
+
+    err = git_refdb_compress(refdb);
+
+    git_refdb_free(refdb);
+    if (err < 0)
+        return Error_set(err);
+    Py_RETURN_NONE;
+}
+
 PyDoc_STRVAR(Repository_status__doc__,
   "status() -> {str: int}\n"
   "\n"
@@ -1540,7 +1468,7 @@ Repository_status(Repository *self)
             path = entry->head_to_index->old_file.path;
         else
             path = entry->index_to_workdir->old_file.path;
-        status = PyInt_FromLong((long) entry->status);
+        status = PyLong_FromLong((long) entry->status);
 
         err = PyDict_SetItemString(dict, path, status);
         Py_CLEAR(status);
@@ -1582,7 +1510,7 @@ Repository_status_file(Repository *self, PyObject *value)
         free(path);
         return err_obj;
     }
-    return PyInt_FromLong(status);
+    return PyLong_FromLong(status);
 }
 
 
@@ -1612,6 +1540,7 @@ Repository_TreeBuilder(Repository *self, PyObject *args)
                 /* return Error_set(GIT_EINVALIDARGS); */
                 return Error_set(GIT_ERROR);
             }
+            if (Object__load((Object*)py_tree) == NULL) { return NULL; } // Lazy load
             tree = py_tree->tree;
         } else {
             err = py_oid_to_git_oid_expand(self->repo, py_src, &oid);
@@ -1656,6 +1585,21 @@ Repository_default_signature__get__(Repository *self)
     return build_signature(NULL, sig, "utf-8");
 }
 
+PyDoc_STRVAR(Repository_odb__doc__, "Return the object database for this repository");
+
+PyObject *
+Repository_odb__get__(Repository *self)
+{
+    git_odb *odb;
+    int err;
+
+    err = git_repository_odb(&odb, self->repo);
+    if (err < 0)
+        return Error_set(err);
+
+    return wrap_odb(odb);
+}
+
 PyDoc_STRVAR(Repository__pointer__doc__, "Get the repo's pointer. For internal use only.");
 PyObject *
 Repository__pointer__get__(Repository *self)
@@ -1747,7 +1691,7 @@ Repository_lookup_note(Repository *self, PyObject* args)
     if (err < 0)
         return Error_set(err);
 
-    return (PyObject*) wrap_note(self, &annotated_id, ref);
+    return (PyObject*) wrap_note(self, NULL, &annotated_id, ref);
 }
 
 PyDoc_STRVAR(Repository_reset__doc__,
@@ -1829,21 +1773,27 @@ Repository_expand_id(Repository *self, PyObject *py_hex)
 }
 
 PyDoc_STRVAR(Repository_add_worktree__doc__,
-    "add_worktree(name, path)\n"
+    "add_worktree(name, path, [ref])\n"
     "\n"
-    "Create a new worktree for this repository.");
+    "Create a new worktree for this repository. If ref is specified, no new \
+    branch will be created and the provided ref will be checked out instead.");
 PyObject *
 Repository_add_worktree(Repository *self, PyObject *args)
 {
     char *c_name;
     char *c_path;
+    Reference *py_reference = NULL;
     git_worktree *wt;
-    int err;
     git_worktree_add_options add_opts = GIT_WORKTREE_ADD_OPTIONS_INIT;
+    
+    int err;
 
-    if (!PyArg_ParseTuple(args, "ss", &c_name, &c_path))
+    if (!PyArg_ParseTuple(args, "ss|O!", &c_name, &c_path, &ReferenceType, &py_reference))
         return NULL;
 
+    if(py_reference != NULL)
+        add_opts.ref = py_reference->reference;
+    
     err = git_worktree_add(&wt, self->repo, c_name, c_path, &add_opts);
     if (err < 0)
         return Error_set(err);
@@ -1909,6 +1859,29 @@ out:
     return py_result;
 }
 
+PyDoc_STRVAR(Repository_apply__doc__,
+  "apply(id)\n"
+  "\n"
+  "Applies the given id into HEAD.\n"
+  "\n"
+  "Applies a diff into HEAD, writing the results into the\n"
+  "working directory.");
+
+PyObject *
+Repository_apply(Repository *self, PyObject *py_diff)
+{
+    int err;
+    git_apply_location_t location = GIT_APPLY_LOCATION_WORKDIR;
+    git_apply_options options = GIT_APPLY_OPTIONS_INIT;
+
+    err = git_apply(self->repo, ((Diff*)py_diff)->diff, location, &options);
+
+    if (err < 0)
+        return Error_set(err);
+
+    Py_RETURN_NONE;
+}
+
 PyMethodDef Repository_methods[] = {
     METHOD(Repository, create_blob, METH_VARARGS),
     METHOD(Repository, create_blob_fromworkdir, METH_VARARGS),
@@ -1920,18 +1893,19 @@ PyMethodDef Repository_methods[] = {
     METHOD(Repository, walk, METH_VARARGS),
     METHOD(Repository, descendant_of, METH_VARARGS),
     METHOD(Repository, merge_base, METH_VARARGS),
-    METHOD(Repository, merge_analysis, METH_O),
+    METHOD(Repository, merge_analysis, METH_VARARGS),
     METHOD(Repository, merge, METH_O),
     METHOD(Repository, cherrypick, METH_O),
-    METHOD(Repository, read, METH_O),
-    METHOD(Repository, write, METH_VARARGS),
+    METHOD(Repository, apply, METH_O),
     METHOD(Repository, create_reference_direct, METH_VARARGS),
     METHOD(Repository, create_reference_symbolic, METH_VARARGS),
+    METHOD(Repository, compress_references, METH_NOARGS),
     METHOD(Repository, listall_references, METH_NOARGS),
     METHOD(Repository, listall_reference_objects, METH_NOARGS),
     METHOD(Repository, listall_submodules, METH_NOARGS),
     METHOD(Repository, init_submodules, METH_VARARGS | METH_KEYWORDS),
     METHOD(Repository, lookup_reference, METH_O),
+    METHOD(Repository, lookup_reference_dwim, METH_O),
     METHOD(Repository, revparse_single, METH_O),
     METHOD(Repository, status, METH_NOARGS),
     METHOD(Repository, status_file, METH_O),
@@ -1963,6 +1937,7 @@ PyGetSetDef Repository_getseters[] = {
     GETTER(Repository, is_bare),
     GETSET(Repository, workdir),
     GETTER(Repository, default_signature),
+    GETTER(Repository, odb),
     GETTER(Repository, _pointer),
     {NULL}
 };
@@ -2001,7 +1976,7 @@ PyTypeObject RepositoryType = {
     (inquiry)Repository_clear,                 /* tp_clear          */
     0,                                         /* tp_richcompare    */
     0,                                         /* tp_weaklistoffset */
-    (getiterfunc)Repository_as_iter,           /* tp_iter           */
+    0,                                         /* tp_iter           */
     0,                                         /* tp_iternext       */
     Repository_methods,                        /* tp_methods        */
     0,                                         /* tp_members        */
