@@ -38,15 +38,9 @@ PyObject *wrap_repository(git_repository *c_repo);
 int  Repository_init(Repository *self, PyObject *args, PyObject *kwds);
 int  Repository_traverse(Repository *self, visitproc visit, void *arg);
 int  Repository_clear(Repository *self);
-int  Repository_contains(Repository *self, PyObject *value);
-
-git_odb_object*
-Repository_read_raw(git_repository *repo, const git_oid *oid, size_t len);
 
 PyObject* Repository_head(Repository *self);
 PyObject* Repository_getitem(Repository *self, PyObject *value);
-PyObject* Repository_read(Repository *self, PyObject *py_hex);
-PyObject* Repository_write(Repository *self, PyObject *args);
 PyObject* Repository_get_index(Repository *self, void *closure);
 PyObject* Repository_get_path(Repository *self, void *closure);
 PyObject* Repository_get_workdir(Repository *self, void *closure);
@@ -78,5 +72,7 @@ PyObject* Repository_blame(Repository *self, PyObject *args, PyObject *kwds);
 
 PyObject* Repository_merge(Repository *self, PyObject *py_oid);
 PyObject* Repository_cherrypick(Repository *self, PyObject *py_oid);
+PyObject* Repository_apply(Repository *self, PyObject *py_diff);
+PyObject* Repository_merge_analysis(Repository *self, PyObject *args);
 
 #endif
