@@ -31,237 +31,51 @@
 #include "error.h"
 #include "utils.h"
 #include "repository.h"
+#include "object.h"
 #include "oid.h"
 #include "tree.h"
 #include "diff.h"
 
 extern PyTypeObject TreeType;
-extern PyTypeObject TreeEntryType;
 extern PyTypeObject DiffType;
 extern PyTypeObject TreeIterType;
 extern PyTypeObject IndexType;
 
-void
-TreeEntry_dealloc(TreeEntry *self)
-{
-    git_tree_entry_free((git_tree_entry*)self->entry);
-    PyObject_Del(self);
-}
-
-
-PyDoc_STRVAR(TreeEntry_filemode__doc__, "Filemode.");
 
 PyObject *
-TreeEntry_filemode__get__(TreeEntry *self)
+treeentry_to_object(const git_tree_entry *entry, Repository *repo)
 {
-    return PyInt_FromLong(git_tree_entry_filemode(self->entry));
-}
-
-
-PyDoc_STRVAR(TreeEntry_name__doc__, "Name.");
-
-PyObject *
-TreeEntry_name__get__(TreeEntry *self)
-{
-    return to_path(git_tree_entry_name(self->entry));
-}
-
-
-PyDoc_STRVAR(TreeEntry__name__doc__, "Name (bytes).");
-
-PyObject *
-TreeEntry__name__get__(TreeEntry *self)
-{
-    return PyBytes_FromString(git_tree_entry_name(self->entry));
-}
-
-
-PyDoc_STRVAR(TreeEntry_type__doc__, "Type.");
-
-PyObject *
-TreeEntry_type__get__(TreeEntry *self)
-{
-    return to_path(git_object_type2string(git_tree_entry_type(self->entry)));
-}
-
-
-PyDoc_STRVAR(TreeEntry_id__doc__, "Object id.");
-
-PyObject *
-TreeEntry_id__get__(TreeEntry *self)
-{
-    const git_oid *oid;
-
-    oid = git_tree_entry_id(self->entry);
-    return git_oid_to_python(oid);
-}
-
-PyDoc_STRVAR(TreeEntry_oid__doc__, "Object id.\n"
-    "This attribute is deprecated. Please use 'id'");
-
-PyObject *
-TreeEntry_oid__get__(TreeEntry *self)
-{
-    return TreeEntry_id__get__(self);
-}
-
-static int
-compare_ids(TreeEntry *a, TreeEntry *b)
-{
-    const git_oid *id_a, *id_b;
-    id_a = git_tree_entry_id(a->entry);
-    id_b = git_tree_entry_id(b->entry);
-    return git_oid_cmp(id_a, id_b);
-}
-
-PyObject *
-TreeEntry_richcompare(PyObject *a, PyObject *b, int op)
-{
-    PyObject *res;
-    TreeEntry *ta, *tb;
-    int cmp;
-
-    /* We only support comparing to another tree entry */
-    if (!PyObject_TypeCheck(b, &TreeEntryType)) {
-        Py_INCREF(Py_NotImplemented);
-        return Py_NotImplemented;
+    if (repo == NULL) {
+        PyErr_SetString(PyExc_ValueError, "expected repository");
+        return NULL;
     }
 
-    ta = (TreeEntry *) a;
-    tb = (TreeEntry *) b;
-
-    /* This is sorting order, if they sort equally, we still need to compare the ids */
-    cmp = git_tree_entry_cmp(ta->entry, tb->entry);
-    if (cmp == 0)
-        cmp = compare_ids(ta, tb);
-
-    switch (op) {
-        case Py_LT:
-            res = (cmp <= 0) ? Py_True: Py_False;
-            break;
-        case Py_LE:
-            res = (cmp < 0) ? Py_True: Py_False;
-            break;
-        case Py_EQ:
-            res = (cmp == 0) ? Py_True: Py_False;
-            break;
-        case Py_NE:
-            res = (cmp != 0) ? Py_True: Py_False;
-            break;
-        case Py_GT:
-            res = (cmp > 0) ? Py_True: Py_False;
-            break;
-        case Py_GE:
-            res = (cmp >= 0) ? Py_True: Py_False;
-            break;
-        default:
-            PyErr_Format(PyExc_RuntimeError, "Unexpected '%d' op", op);
-            return NULL;
-    }
-
-    Py_INCREF(res);
-    return res;
+    return wrap_object(NULL, repo, entry);
 }
 
-
-PyDoc_STRVAR(TreeEntry_hex__doc__, "Hex oid.");
-
-PyObject *
-TreeEntry_hex__get__(TreeEntry *self)
-{
-    return git_oid_to_py_str(git_tree_entry_id(self->entry));
-}
-
-PyObject *
-TreeEntry_repr(TreeEntry *self)
-{
-    char str[GIT_OID_HEXSZ + 1] = { 0 };
-    const char *typename;
-
-    typename = git_object_type2string(git_tree_entry_type(self->entry));
-    git_oid_fmt(str, git_tree_entry_id(self->entry));
-    return PyString_FromFormat("pygit2.TreeEntry('%s', %s, %s)", git_tree_entry_name(self->entry), typename, str);
-}
-
-PyGetSetDef TreeEntry_getseters[] = {
-    GETTER(TreeEntry, filemode),
-    GETTER(TreeEntry, name),
-    GETTER(TreeEntry, _name),
-    GETTER(TreeEntry, oid),
-    GETTER(TreeEntry, id),
-    GETTER(TreeEntry, hex),
-    GETTER(TreeEntry, type),
-    {NULL}
-};
-
-PyDoc_STRVAR(TreeEntry__doc__, "TreeEntry objects.");
-
-PyTypeObject TreeEntryType = {
-    PyVarObject_HEAD_INIT(NULL, 0)
-    "_pygit2.TreeEntry",                       /* tp_name           */
-    sizeof(TreeEntry),                         /* tp_basicsize      */
-    0,                                         /* tp_itemsize       */
-    (destructor)TreeEntry_dealloc,             /* tp_dealloc        */
-    0,                                         /* tp_print          */
-    0,                                         /* tp_getattr        */
-    0,                                         /* tp_setattr        */
-    0,                                         /* tp_compare        */
-    (reprfunc)TreeEntry_repr,                  /* tp_repr           */
-    0,                                         /* tp_as_number      */
-    0,                                         /* tp_as_sequence    */
-    0,                                         /* tp_as_mapping     */
-    0,                                         /* tp_hash           */
-    0,                                         /* tp_call           */
-    0,                                         /* tp_str            */
-    0,                                         /* tp_getattro       */
-    0,                                         /* tp_setattro       */
-    0,                                         /* tp_as_buffer      */
-    Py_TPFLAGS_DEFAULT,                        /* tp_flags          */
-    TreeEntry__doc__,                          /* tp_doc            */
-    0,                                         /* tp_traverse       */
-    0,                                         /* tp_clear          */
-    (richcmpfunc)TreeEntry_richcompare,        /* tp_richcompare    */
-    0,                                         /* tp_weaklistoffset */
-    0,                                         /* tp_iter           */
-    0,                                         /* tp_iternext       */
-    0,                                         /* tp_methods        */
-    0,                                         /* tp_members        */
-    TreeEntry_getseters,                       /* tp_getset         */
-    0,                                         /* tp_base           */
-    0,                                         /* tp_dict           */
-    0,                                         /* tp_descr_get      */
-    0,                                         /* tp_descr_set      */
-    0,                                         /* tp_dictoffset     */
-    0,                                         /* tp_init           */
-    0,                                         /* tp_alloc          */
-    0,                                         /* tp_new            */
-};
-
 Py_ssize_t
 Tree_len(Tree *self)
 {
-    assert(self->tree);
+    if (Object__load((Object*)self) == NULL) { return -1; } // Lazy load
     return (Py_ssize_t)git_tree_entrycount(self->tree);
 }
 
 int
 Tree_contains(Tree *self, PyObject *py_name)
 {
-    int err;
-    git_tree_entry *entry;
-    char *name;
+    if (Object__load((Object*)self) == NULL) { return -1; } // Lazy load
 
-    name = py_path_to_c_str(py_name);
+    char *name = py_path_to_c_str(py_name);
     if (name == NULL)
         return -1;
 
-    err = git_tree_entry_bypath(&entry, self->tree, name);
+    git_tree_entry *entry;
+    int err = git_tree_entry_bypath(&entry, self->tree, name);
     free(name);
 
-    if (err == GIT_ENOTFOUND)
+    if (err == GIT_ENOTFOUND) {
         return 0;
-
-    if (err < 0) {
+    } else if (err < 0) {
         Error_set(err);
         return -1;
     }
@@ -271,30 +85,18 @@ Tree_contains(Tree *self, PyObject *py_name)
     return 1;
 }
 
-TreeEntry *
-wrap_tree_entry(const git_tree_entry *entry)
-{
-    TreeEntry *py_entry;
-
-    py_entry = PyObject_New(TreeEntry, &TreeEntryType);
-    if (py_entry)
-        py_entry->entry = entry;
-
-    return py_entry;
-}
-
 int
-Tree_fix_index(Tree *self, PyObject *py_index)
+Tree_fix_index(const git_tree *tree, PyObject *py_index)
 {
     long index;
     size_t len;
     long slen;
 
-    index = PyInt_AsLong(py_index);
+    index = PyLong_AsLong(py_index);
     if (PyErr_Occurred())
         return -1;
 
-    len = git_tree_entrycount(self->tree);
+    len = git_tree_entrycount(tree);
     slen = (long)len;
     if (index >= slen) {
         PyErr_SetObject(PyExc_IndexError, py_index);
@@ -326,18 +128,18 @@ Tree_iter(Tree *self)
     return (PyObject*)iter;
 }
 
-TreeEntry *
-Tree_getitem_by_index(Tree *self, PyObject *py_index)
+PyObject*
+tree_getentry_by_index(const git_tree *tree, Repository *repo, PyObject *py_index)
 {
     int index;
     const git_tree_entry *entry_src;
     git_tree_entry *entry;
 
-    index = Tree_fix_index(self, py_index);
+    index = Tree_fix_index(tree, py_index);
     if (PyErr_Occurred())
         return NULL;
 
-    entry_src = git_tree_entry_byindex(self->tree, index);
+    entry_src = git_tree_entry_byindex(tree, index);
     if (!entry_src) {
         PyErr_SetObject(PyExc_IndexError, py_index);
         return NULL;
@@ -348,38 +150,55 @@ Tree_getitem_by_index(Tree *self, PyObject *py_index)
         return NULL;
     }
 
-    return wrap_tree_entry(entry);
+    return treeentry_to_object(entry, repo);
 }
 
-TreeEntry *
-Tree_getitem(Tree *self, PyObject *value)
+PyObject*
+tree_getentry_by_path(const git_tree *tree, Repository *repo, PyObject *py_path)
 {
     char *path;
     git_tree_entry *entry;
     int err;
 
-    /* Case 1: integer */
-    if (PyInt_Check(value))
-        return Tree_getitem_by_index(self, value);
-
-    /* Case 2: byte or text string */
-    path = py_path_to_c_str(value);
-    if (path == NULL)
+    path = py_path_to_c_str(py_path);
+    if (path == NULL) {
+        PyErr_SetString(PyExc_TypeError, "Value must be a path string");
         return NULL;
+    }
 
-    err = git_tree_entry_bypath(&entry, self->tree, path);
+    err = git_tree_entry_bypath(&entry, tree, path);
     free(path);
 
     if (err == GIT_ENOTFOUND) {
-        PyErr_SetObject(PyExc_KeyError, value);
+        PyErr_SetObject(PyExc_KeyError, py_path);
         return NULL;
     }
 
     if (err < 0)
-        return (TreeEntry*)Error_set(err);
+        return Error_set(err);
 
     /* git_tree_entry_dup is already done in git_tree_entry_bypath */
-    return wrap_tree_entry(entry);
+    return treeentry_to_object(entry, repo);
+}
+
+PyObject*
+Tree_subscript(Tree *self, PyObject *value)
+{
+    if (Object__load((Object*)self) == NULL) { return NULL; } // Lazy load
+
+    /* Case 1: integer */
+    if (PyLong_Check(value))
+        return tree_getentry_by_index(self->tree, self->repo, value);
+
+    /* Case 2: byte or text string */
+    return tree_getentry_by_path(self->tree, self->repo, value);
+}
+
+PyObject *
+Tree_divide(Tree *self, PyObject *value)
+{
+    if (Object__load((Object*)self) == NULL) { return NULL; } // Lazy load
+    return tree_getentry_by_path(self->tree, self->repo, value);
 }
 
 
@@ -406,19 +225,19 @@ Tree_diff_to_workdir(Tree *self, PyObject *args)
 {
     git_diff_options opts = GIT_DIFF_OPTIONS_INIT;
     git_diff *diff;
-    Repository *py_repo;
     int err;
 
     if (!PyArg_ParseTuple(args, "|IHH", &opts.flags, &opts.context_lines,
                                         &opts.interhunk_lines))
         return NULL;
 
-    py_repo = self->repo;
-    err = git_diff_tree_to_workdir(&diff, py_repo->repo, self->tree, &opts);
+    if (Object__load((Object*)self) == NULL) { return NULL; } // Lazy load
+
+    err = git_diff_tree_to_workdir(&diff, self->repo->repo, self->tree, &opts);
     if (err < 0)
         return Error_set(err);
 
-    return wrap_diff(diff, py_repo);
+    return wrap_diff(diff, self->repo);
 }
 
 
@@ -451,7 +270,6 @@ Tree_diff_to_index(Tree *self, PyObject *args, PyObject *kwds)
     git_index *index;
     char *buffer;
     Py_ssize_t length;
-    Repository *py_repo;
     PyObject *py_idx, *py_idx_ptr;
     int err;
 
@@ -485,12 +303,13 @@ Tree_diff_to_index(Tree *self, PyObject *args, PyObject *kwds)
     /* the "buffer" contains the pointer */
     index = *((git_index **) buffer);
 
-    py_repo = self->repo;
-    err = git_diff_tree_to_index(&diff, py_repo->repo, self->tree, index, &opts);
+    if (Object__load((Object*)self) == NULL) { return NULL; } // Lazy load
+
+    err = git_diff_tree_to_index(&diff, self->repo->repo, self->tree, index, &opts);
     if (err < 0)
         return Error_set(err);
 
-    return wrap_diff(diff, py_repo);
+    return wrap_diff(diff, self->repo);
 }
 
 
@@ -524,34 +343,38 @@ Tree_diff_to_tree(Tree *self, PyObject *args, PyObject *kwds)
 {
     git_diff_options opts = GIT_DIFF_OPTIONS_INIT;
     git_diff *diff;
-    git_tree *from, *to, *tmp;
-    Repository *py_repo;
+    git_tree *from, *to = NULL, *tmp;
     int err, swap = 0;
     char *keywords[] = {"obj", "flags", "context_lines", "interhunk_lines",
                         "swap", NULL};
 
-    Tree *py_tree = NULL;
+    Tree *other = NULL;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O!IHHi", keywords,
-                                     &TreeType, &py_tree, &opts.flags,
+                                     &TreeType, &other, &opts.flags,
                                      &opts.context_lines,
                                      &opts.interhunk_lines, &swap))
         return NULL;
 
-    py_repo = self->repo;
-    to = (py_tree == NULL) ? NULL : py_tree->tree;
+    if (Object__load((Object*)self) == NULL) { return NULL; } // Lazy load
     from = self->tree;
+
+    if (other) {
+        if (Object__load((Object*)other) == NULL) { return NULL; } // Lazy load
+        to = other->tree;
+    }
+
     if (swap > 0) {
         tmp = from;
         from = to;
         to = tmp;
     }
 
-    err = git_diff_tree_to_tree(&diff, py_repo->repo, from, to, &opts);
+    err = git_diff_tree_to_tree(&diff, self->repo->repo, from, to, &opts);
     if (err < 0)
         return Error_set(err);
 
-    return wrap_diff(diff, py_repo);
+    return wrap_diff(diff, self->repo);
 }
 
 
@@ -568,7 +391,7 @@ PySequenceMethods Tree_as_sequence = {
 
 PyMappingMethods Tree_as_mapping = {
     (lenfunc)Tree_len,            /* mp_length */
-    (binaryfunc)Tree_getitem,     /* mp_subscript */
+    (binaryfunc)Tree_subscript,   /* mp_subscript */
     0,                            /* mp_ass_subscript */
 };
 
@@ -579,6 +402,48 @@ PyMethodDef Tree_methods[] = {
     {NULL}
 };
 
+/* Py2/3 compatible structure
+ * see https://py3c.readthedocs.io/en/latest/ext-types.html#pynumbermethods
+ */
+PyNumberMethods Tree_as_number = {
+    0,                          /* nb_add */
+    0,                          /* nb_subtract */
+    0,                          /* nb_multiply */
+    0,                          /* nb_remainder */
+    0,                          /* nb_divmod */
+    0,                          /* nb_power */
+    0,                          /* nb_negative */
+    0,                          /* nb_positive */
+    0,                          /* nb_absolute */
+    0,                          /* nb_bool (Py2: nb_nonzero) */
+    0,                          /* nb_invert */
+    0,                          /* nb_lshift */
+    0,                          /* nb_rshift */
+    0,                          /* nb_and */
+    0,                          /* nb_xor */
+    0,                          /* nb_or */
+    0,                          /* nb_int */
+    0,                          /* nb_reserved (Py2: nb_long) */
+    0,                          /* nb_float */
+    0,                          /* nb_inplace_add */
+    0,                          /* nb_inplace_subtract */
+    0,                          /* nb_inplace_multiply */
+    0,                          /* nb_inplace_remainder */
+    0,                          /* nb_inplace_power */
+    0,                          /* nb_inplace_lshift */
+    0,                          /* nb_inplace_rshift */
+    0,                          /* nb_inplace_and */
+    0,                          /* nb_inplace_xor */
+    0,                          /* nb_inplace_or */
+    0,                          /* nb_floor_divide */
+    (binaryfunc)Tree_divide,    /* nb_true_divide */
+    0,                          /* nb_inplace_floor_divide */
+    0,                          /* nb_inplace_true_divide */
+    0,                          /* nb_index */
+// Python 3.5+
+//  0,                          /* nb_matrix_multiply */
+//  0,                          /* nb_inplace_matrix_multiply */
+};
 
 PyDoc_STRVAR(Tree__doc__, "Tree objects.");
 
@@ -593,7 +458,7 @@ PyTypeObject TreeType = {
     0,                                         /* tp_setattr        */
     0,                                         /* tp_compare        */
     0,                                         /* tp_repr           */
-    0,                                         /* tp_as_number      */
+    &Tree_as_number,                           /* tp_as_number      */
     &Tree_as_sequence,                         /* tp_as_sequence    */
     &Tree_as_mapping,                          /* tp_as_mapping     */
     0,                                         /* tp_hash           */
@@ -602,7 +467,7 @@ PyTypeObject TreeType = {
     0,                                         /* tp_getattro       */
     0,                                         /* tp_setattro       */
     0,                                         /* tp_as_buffer      */
-    Py_TPFLAGS_DEFAULT,                        /* tp_flags          */
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,  /* tp_flags */
     Tree__doc__,                               /* tp_doc            */
     0,                                         /* tp_traverse       */
     0,                                         /* tp_clear          */
@@ -631,7 +496,7 @@ TreeIter_dealloc(TreeIter *self)
     PyObject_Del(self);
 }
 
-TreeEntry *
+PyObject*
 TreeIter_iternext(TreeIter *self)
 {
     const git_tree_entry *entry_src;
@@ -647,7 +512,8 @@ TreeIter_iternext(TreeIter *self)
         PyErr_SetNone(PyExc_MemoryError);
         return NULL;
     }
-    return wrap_tree_entry(entry);
+
+    return treeentry_to_object(entry, self->owner->repo);
 }
 
 
