@@ -32,8 +32,8 @@
 #include <Python.h>
 #include <git2.h>
 
-#if !(LIBGIT2_VER_MAJOR == 0 && LIBGIT2_VER_MINOR == 27)
-#error You need a compatible libgit2 version (v0.27.x)
+#if !(LIBGIT2_VER_MAJOR == 0 && LIBGIT2_VER_MINOR == 28)
+#error You need a compatible libgit2 version (v0.28.x)
 #endif
 
 /*
@@ -56,6 +56,24 @@ typedef struct {
     git_oid oid;
 } Oid;
 
+typedef struct {
+    PyObject_HEAD
+    git_odb *odb;
+} Odb;
+
+typedef struct {
+    PyObject_HEAD
+    git_odb_backend *odb_backend;
+} OdbBackend;
+
+typedef struct {
+    OdbBackend super;
+} OdbBackendPack;
+
+typedef struct {
+    OdbBackend super;
+} OdbBackendLoose;
+
 
 #define SIMPLE_TYPE(_name, _ptr_type, _ptr_name) \
         typedef struct {\
@@ -64,30 +82,35 @@ typedef struct {
             _ptr_type *_ptr_name;\
         } _name;
 
+#define OBJECT_TYPE(_name, _ptr_type, _ptr_name) \
+        typedef struct {\
+            PyObject_HEAD\
+            Repository *repo;\
+            _ptr_type *_ptr_name;\
+            const git_tree_entry *entry;\
+        } _name;
+
 
 /* git object types
  *
  * The structs for some of the object subtypes are identical except for
  * the type of their object pointers. */
-SIMPLE_TYPE(Object, git_object, obj)
-SIMPLE_TYPE(Commit, git_commit, commit)
-SIMPLE_TYPE(Tree, git_tree, tree)
-SIMPLE_TYPE(Blob, git_blob, blob)
-SIMPLE_TYPE(Tag, git_tag, tag)
+OBJECT_TYPE(Object, git_object, obj)
+OBJECT_TYPE(Commit, git_commit, commit)
+OBJECT_TYPE(Tree, git_tree, tree)
+OBJECT_TYPE(Blob, git_blob, blob)
+OBJECT_TYPE(Tag, git_tag, tag)
 
-/* git_worktree */
-typedef struct {
-    PyObject_HEAD
-    Repository *repo;
-    git_worktree *worktree;
-} Worktree;
+SIMPLE_TYPE(Worktree, git_worktree, worktree)
 
 /* git_note */
 typedef struct {
     PyObject_HEAD
     Repository *repo;
+    const char *ref;
+    PyObject *annotated_id;
+    PyObject *id;
     git_note *note;
-    PyObject* annotated_id;
 } Note;
 
 typedef struct {
@@ -161,11 +184,6 @@ SIMPLE_TYPE(DiffStats, git_diff_stats, stats);
 /* git_tree_walk , git_treebuilder*/
 SIMPLE_TYPE(TreeBuilder, git_treebuilder, bld)
 
-typedef struct {
-    PyObject_HEAD
-    const git_tree_entry *entry;
-} TreeEntry;
-
 typedef struct {
     PyObject_HEAD
     Tree *owner;
@@ -213,4 +231,10 @@ typedef struct {
     char *encoding;
 } Signature;
 
+/* git_mailmap */
+typedef struct {
+    PyObject_HEAD
+    git_mailmap *mailmap;
+} Mailmap;
+
 #endif
