@@ -28,9 +28,10 @@
 #define PY_SSIZE_T_CLEAN
 #include <Python.h>
 #include "error.h"
-#include "utils.h"
+#include "object.h"
 #include "oid.h"
 #include "tree.h"
+#include "utils.h"
 #include "walker.h"
 
 extern PyTypeObject CommitType;
@@ -100,7 +101,7 @@ Walker_sort(Walker *self, PyObject *py_sort_mode)
 {
     long sort_mode;
 
-    sort_mode = PyInt_AsLong(py_sort_mode);
+    sort_mode = PyLong_AsLong(py_sort_mode);
     if (sort_mode == -1 && PyErr_Occurred())
         return NULL;
 
@@ -146,7 +147,6 @@ Walker_iternext(Walker *self)
 {
     int err;
     git_commit *commit;
-    Commit *py_commit;
     git_oid oid;
 
     err = git_revwalk_next(&oid, self->walk);
@@ -157,13 +157,7 @@ Walker_iternext(Walker *self)
     if (err < 0)
         return Error_set(err);
 
-    py_commit = PyObject_New(Commit, &CommitType);
-    if (py_commit) {
-        py_commit->commit = commit;
-        Py_INCREF(self->repo);
-        py_commit->repo = self->repo;
-    }
-    return (PyObject*)py_commit;
+    return wrap_object((git_object*)commit, self->repo, NULL);
 }
 
 PyMethodDef Walker_methods[] = {
