@@ -33,15 +33,6 @@
 #include <git2.h>
 #include <git2/worktree.h>
 
-struct git_worktree {
-	char *name;
-	char *gitlink_path;
-	char *gitdir_path;
-	char *commondir_path;
-	char *parent_path;
-	int locked:1;
-};
-
 PyObject* wrap_worktree(Repository* repo, git_worktree* wt);
 
 #endif
