@@ -1,5 +1,3 @@
-# -*- coding: UTF-8 -*-
-#
 # Copyright 2010-2019 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
@@ -25,10 +23,6 @@
 # the Free Software Foundation, 51 Franklin Street, Fifth Floor,
 # Boston, MA 02110-1301, USA.
 
-# The future
-from __future__ import absolute_import
-from __future__ import unicode_literals, print_function
-
 # Standard Library
 from os.path import join
 
@@ -40,6 +34,7 @@ try:
 except ImportError:
     __pypy__ = None
 
+
 class RepositorySignatureTest(utils.RepoTestCase):
 
     def test_no_attr(self):
