@@ -1,5 +1,3 @@
-# -*- coding: UTF-8 -*-
-#
 # Copyright 2010-2019 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
@@ -27,14 +25,12 @@
 
 """Tests for Blame objects."""
 
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
 import pytest
 
 from pygit2 import Signature, Oid
 from . import utils
 
+
 PATH = 'hello.txt'
 
 HUNKS = [
