@@ -1,5 +1,3 @@
-# -*- coding: UTF-8 -*-
-#
 # Copyright 2010-2019 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
@@ -27,9 +25,6 @@
 
 """Tests for Commit objects."""
 
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
 import unittest
 import sys
 
@@ -42,10 +37,9 @@ from . import utils
 # we need to check and change the test accordingly
 try:
     import __pypy__
-    pypy2 =  sys.version_info[0] < 3
 except ImportError:
     __pypy__ = None
-    pypy2 = False
+
 
 COMMIT_SHA = '5fe808e8953c12735680c257f56600cb0de44b10'
 
@@ -144,12 +138,11 @@ class CommitTest(utils.BareRepoTestCase):
 
         commit = self.repo[COMMIT_SHA]
 
-        error_type = AttributeError if not pypy2 else TypeError
-        with pytest.raises(error_type): setattr(commit, 'message', message)
-        with pytest.raises(error_type): setattr(commit, 'committer', committer)
-        with pytest.raises(error_type): setattr(commit, 'author', author)
-        with pytest.raises(error_type): setattr(commit, 'tree', None)
-        with pytest.raises(error_type): setattr(commit, 'parents', None)
+        with pytest.raises(AttributeError): setattr(commit, 'message', message)
+        with pytest.raises(AttributeError): setattr(commit, 'committer', committer)
+        with pytest.raises(AttributeError): setattr(commit, 'author', author)
+        with pytest.raises(AttributeError): setattr(commit, 'tree', None)
+        with pytest.raises(AttributeError): setattr(commit, 'parents', None)
 
 
 class GpgSignatureTestCase(utils.GpgSignedRepoTestCase):
