@@ -1,5 +1,3 @@
-# -*- coding: UTF-8 -*-
-#
 # Copyright 2010-2019 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
@@ -27,9 +25,6 @@
 
 """Tests for Diff objects."""
 
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
 from itertools import chain
 import textwrap
 
@@ -189,7 +184,6 @@ class DiffTest(utils.BareRepoTestCase):
 
         def _test(diff):
             assert diff is not None
-            # self.assertIn is 2.7 only
             assert 2 == sum(map(lambda x: len(x.hunks), diff))
 
             patch = diff[0]
