@@ -1,5 +1,3 @@
-# -*- coding: UTF-8 -*-
-#
 # Copyright 2010-2019 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
@@ -27,9 +25,6 @@
 
 """Tests for Index files."""
 
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
 import os
 
 import pytest
@@ -177,6 +172,15 @@ class IndexTest(utils.RepoTestCase):
         index.remove('hello.txt')
         assert 'hello.txt' not in index
 
+    def test_remove_all(self):
+        index = self.repo.index
+        print([i.path for i in index])
+        assert 'hello.txt' in index
+        index.remove_all(['*.txt'])
+        assert 'hello.txt' not in index
+
+        index.remove_all(['not-existing'])  # this doesn't error
+
     def test_change_attributes(self):
         index = self.repo.index
         entry = index['hello.txt']
@@ -191,6 +195,7 @@ class IndexTest(utils.RepoTestCase):
         assert pygit2.GIT_FILEMODE_BLOB_EXECUTABLE == entry.mode
 
     def test_write_tree_to(self):
+        pygit2.option(pygit2.GIT_OPT_ENABLE_STRICT_OBJECT_CREATION, False)
         with utils.TemporaryRepository(('tar', 'emptyrepo')) as path:
             nrepo = Repository(path)
             id = self.repo.index.write_tree(nrepo)
