@@ -1,5 +1,3 @@
-# -*- coding: UTF-8 -*-
-#
 # Copyright 2010-2019 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
@@ -27,10 +25,6 @@
 
 """Tests for merging and information about it."""
 
-# Import from the future
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
 import os
 
 import pytest
@@ -41,6 +35,7 @@ import pygit2
 
 from . import utils
 
+
 class MergeTestBasic(utils.RepoTestCaseForMerging):
 
     def test_merge_none(self):
@@ -49,8 +44,13 @@ class MergeTestBasic(utils.RepoTestCaseForMerging):
     def test_merge_analysis_uptodate(self):
         branch_head_hex = '5ebeeebb320790caf276b9fc8b24546d63316533'
         branch_id = self.repo.get(branch_head_hex).id
-        analysis, preference = self.repo.merge_analysis(branch_id)
 
+        analysis, preference = self.repo.merge_analysis(branch_id)
+        assert analysis & GIT_MERGE_ANALYSIS_UP_TO_DATE
+        assert not analysis & GIT_MERGE_ANALYSIS_FASTFORWARD
+        assert {} == self.repo.status()
+
+        analysis, preference = self.repo.merge_analysis(branch_id, 'refs/heads/ff-branch')
         assert analysis & GIT_MERGE_ANALYSIS_UP_TO_DATE
         assert not analysis & GIT_MERGE_ANALYSIS_FASTFORWARD
         assert {} == self.repo.status()
@@ -58,11 +58,17 @@ class MergeTestBasic(utils.RepoTestCaseForMerging):
     def test_merge_analysis_fastforward(self):
         branch_head_hex = 'e97b4cfd5db0fb4ebabf4f203979ca4e5d1c7c87'
         branch_id = self.repo.get(branch_head_hex).id
+
         analysis, preference = self.repo.merge_analysis(branch_id)
         assert not analysis & GIT_MERGE_ANALYSIS_UP_TO_DATE
         assert analysis & GIT_MERGE_ANALYSIS_FASTFORWARD
         assert {} == self.repo.status()
 
+        analysis, preference = self.repo.merge_analysis(branch_id, 'refs/heads/master')
+        assert not analysis & GIT_MERGE_ANALYSIS_UP_TO_DATE
+        assert analysis & GIT_MERGE_ANALYSIS_FASTFORWARD
+        assert {} == self.repo.status()
+
     def test_merge_no_fastforward_no_conflicts(self):
         branch_head_hex = '03490f16b15a09913edb3a067a3dc67fbb8d41f1'
         branch_id = self.repo.get(branch_head_hex).id
