@@ -1,5 +1,3 @@
-# -*- coding: UTF-8 -*-
-#
 # Copyright 2010-2019 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
@@ -27,10 +25,6 @@
 
 """Tests for Repository objects."""
 
-# Import from the future
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
 # Import from the Standard Library
 import binascii
 import unittest
@@ -39,14 +33,9 @@ import tempfile
 import os
 from os.path import join, realpath
 import sys
+from urllib.request import pathname2url
 
 import pytest
-import six
-
-if six.PY2:
-    from urllib import pathname2url
-else:
-    from urllib.request import pathname2url
 
 # Import from pygit2
 from pygit2 import GIT_OBJ_ANY, GIT_OBJ_BLOB, GIT_OBJ_COMMIT
@@ -476,6 +465,28 @@ class RepositoryTest_II(utils.RepoTestCase):
         assert revert_diff_stats.deletions == commit_diff_stats.insertions
         assert revert_diff_stats.files_changed == commit_diff_stats.files_changed
 
+    def test_diff_patch(self):
+        new_content = ['bye world', 'adiós', 'au revoir monde']
+        new_content = ''.join(x + os.linesep for x in new_content)
+
+        # create the patch
+        with open(os.path.join(self.repo.workdir, 'hello.txt'), 'wb') as f:
+            f.write(new_content.encode('utf-8'))
+
+        patch = self.repo.diff().patch
+
+        # rollback all changes
+        self.repo.checkout('HEAD', strategy=pygit2.GIT_CHECKOUT_FORCE)
+
+        # apply the patch and compare
+        diff = pygit2.Diff.parse_diff(patch)
+        self.repo.apply(diff)
+
+        with open(os.path.join(self.repo.workdir, 'hello.txt'), 'rb') as f:
+            content = f.read().decode('utf-8')
+
+        self.assertEqual(content, new_content)
+
 
 class RepositorySignatureTest(utils.RepoTestCase):
 
@@ -666,24 +677,14 @@ class WorktreeTestCase(utils.RepoTestCase):
         os.rmdir(worktree_dir)
 
         def _check_worktree(worktree):
-            path = os.path.realpath(
-                os.path.join(worktree_dir, '.git'))
-            git_path = os.path.realpath(
-                os.path.join(self.repo.path, 'worktrees', worktree_name))
-
             # Confirm the name attribute matches the specified name
             assert worktree.name == worktree_name
             # Confirm the path attribute points to the correct path
-            assert os.path.realpath(worktree.path) == path
+            assert os.path.realpath(worktree.path) == worktree_dir
             # The "gitdir" in a worktree should be a file with a reference to
             # the actual gitdir. Let's make sure that the path exists and is a
             # file.
-            assert os.path.isfile(path)
-            # Confirm the git_path attribute points to the correct path
-            assert os.path.realpath(worktree.git_path) == git_path
-            # Confirm the worktree directory in the main checkout's gitdir
-            # actually exists
-            assert os.path.isdir(git_path)
+            assert os.path.isfile(os.path.join(worktree_dir, '.git'))
 
         # We should have zero worktrees
         assert self.repo.list_worktrees() == []
@@ -693,6 +694,8 @@ class WorktreeTestCase(utils.RepoTestCase):
         _check_worktree(worktree)
         # We should have one worktree now
         assert self.repo.list_worktrees() == [worktree_name]
+        # We should also have a branch of the same name
+        assert worktree_name in self.repo.listall_branches()
         # Test that lookup_worktree() returns a properly-instantiated
         # pygit2._Worktree object
         _check_worktree(self.repo.lookup_worktree(worktree_name))
@@ -706,5 +709,36 @@ class WorktreeTestCase(utils.RepoTestCase):
         # something to take up with libgit2.
         worktree.prune(True)
         assert self.repo.list_worktrees() == []
-        # Confirm that the repo's data dir has been removed
-        assert not os.path.isdir(worktree.git_path)
+
+    def test_worktree_custom_ref(self):
+        worktree_name = 'foo'
+        worktree_dir = tempfile.mkdtemp()
+        branch_name = 'version1'
+
+        # New branch based on head
+        tip = self.repo.revparse_single('HEAD')
+        worktree_ref = self.repo.branches.create(branch_name, tip)
+        # Delete temp path so that it's not present when we attempt to add the
+        # worktree later
+        os.rmdir(worktree_dir)
+
+        # Add a worktree for the given ref
+        worktree = self.repo.add_worktree(worktree_name, worktree_dir, worktree_ref)
+        # We should have one worktree now
+        assert self.repo.list_worktrees() == [worktree_name]
+        # We should not have a branch of the same name
+        assert worktree_name not in self.repo.listall_branches()
+
+        # The given ref is checked out in the "worktree repository"
+        assert worktree_ref.is_checked_out()
+
+        # Remove the worktree dir and prune the worktree
+        shutil.rmtree(worktree_dir)
+        worktree.prune(True)
+        assert self.repo.list_worktrees() == []
+
+        # The ref is no longer checked out
+        assert worktree_ref.is_checked_out() == False
+
+        # The branch still exists
+        assert branch_name in self.repo.branches
