@@ -1,5 +1,3 @@
-# -*- coding: UTF-8 -*-
-#
 # Copyright 2010-2019 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
@@ -27,14 +25,12 @@
 
 """Tests for Submodule objects."""
 
-# Import from the future
-from __future__ import absolute_import
-
 import os
 import unittest
 
 from . import utils
 
+
 SUBM_NAME = 'submodule'
 SUBM_PATH = 'submodule'
 SUBM_URL = 'https://github.com/libgit2/pygit2'
