@@ -1,5 +1,3 @@
-# -*- coding: UTF-8 -*-
-#
 # Copyright 2010-2019 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
@@ -25,8 +23,6 @@
 # the Free Software Foundation, 51 Franklin Street, Fifth Floor,
 # Boston, MA 02110-1301, USA.
 
-"""Test utilities for libgit2."""
-
 import gc
 import hashlib
 import os
@@ -133,13 +129,13 @@ class NoRepoTestCase(unittest.TestCase):
 
 class AutoRepoTestCase(NoRepoTestCase):
     def setUp(self):
-        super(AutoRepoTestCase, self).setUp()
+        super().setUp()
         self.repo_ctxtmgr = TemporaryRepository(self.repo_spec)
         self.repo_path = self.repo_ctxtmgr.__enter__()
         self.repo = pygit2.Repository(self.repo_path)
 
     def tearDown(self):
-        super(AutoRepoTestCase, self).tearDown()
+        super().tearDown()
         self.repo_ctxtmgr.__exit__(None, None, None)
 
 
